@@ -26,10 +26,18 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <vector>
+#include <string>
+
 #include "common.h"
 
 int VERBOSE = 0;
 
+void fatal_error(const std::string &s) {
+	fprintf(stderr, "\n\nerror:\n%s\n\n", s.c_str());
+	exit(1);
+}
+
 void fatal_error(const char *fmt, ...) {
 	va_list argp;
 	
@@ -44,16 +52,10 @@ void fatal_error(const char *fmt, ...) {
 	exit(1);
 }
 
-void *_myalloc(size_t size) {
-	if(size <= 0) fatal_error("size <= 0 in MYALLOC (%ld)", (long)size);
-	void *p = malloc(size);
-	if(!p) fatal_error("out of memory (size=%zd)", size);
-	return p;
-}
-
-void *_remyalloc(void *p, size_t size) {
-	if(size <= 0) fatal_error("size <= 0 in REMYALLOC (%ld)", (long)size);
-	p = realloc(p, size);
-	if(!p) fatal_error("out of memory (size=%zd)", size);
-	return p;
+std::vector<std::string> argv_to_list(int argc, char **argv) {
+	std::vector<std::string> ret;
+	for(int i=0; i<argc; i++) {
+		ret.push_back(argv[i]);
+	}
+	return ret;
 }
