@@ -26,30 +26,38 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <limits>
+#include <cassert>
+
 #include "common.h"
 #include "polygon.h"
 #include "debugplot.h"
 
-#define DEBUG 0
+static const bool DEBUG = 0;
 
-static inline double seg_ang(vertex_t v0, vertex_t v1) {
+namespace dangdal {
+
+typedef std::vector<uint8_t> Keeps;
+
+static inline double seg_ang(Vertex v0, Vertex v1) {
 	double dx = v1.x - v0.x;
 	double dy = v1.y - v0.y;
 	return atan2(dy, dx);
 }
 
-static inline double seg_len(vertex_t v0, vertex_t v1) {
+static inline double seg_len(Vertex v0, Vertex v1) {
 	double dx = v1.x - v0.x;
 	double dy = v1.y - v0.y;
 	return sqrt(dx*dx + dy*dy);
 }
 
-static int find_bottom_pt(ring_t *ring) {
-	double min = 0;
-	int min_idx = -1;
-	for(int i=0; i<ring->npts; i++) {
-		double y = ring->pts[i].y;
-		if(i==0 || y<min) {
+static size_t find_bottom_pt(const Ring &ring) {
+	assert(ring.pts.size());
+	double min = ring.pts[0].y;
+	size_t min_idx = 0;
+	for(size_t i=1; i<ring.pts.size(); i++) {
+		double y = ring.pts[i].y;
+		if(y<min) {
 			min = y;
 			min_idx = i;
 		}
@@ -57,17 +65,38 @@ static int find_bottom_pt(ring_t *ring) {
 	return min_idx;
 }
 
-static int find_next_convex(ring_t *ring, int start_idx, int limit_idx, double start_ang, double *ang_out) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
-	vertex_t v0 = pts[start_idx];
+struct FindNextConvexRetval {
+	FindNextConvexRetval(
+		bool _error, size_t _idx, double _ang
+	) : error(_error), idx(_idx), ang(_ang) { }
+
+	FindNextConvexRetval(
+		size_t _idx, double _ang
+	) : error(false), idx(_idx), ang(_ang) { }
+
+	static const FindNextConvexRetval ERROR() {
+		return FindNextConvexRetval(true, 0, 0);
+	}
+
+	bool error;
+	size_t idx;
+	double ang;
+};
+
+static FindNextConvexRetval find_next_convex(
+	const Ring &ring, size_t start_idx, size_t limit_idx, double start_ang
+) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
+	Vertex v0 = pts[start_idx];
 	double min_angdiff = M_PI;
 	double last_ad = -999;
-	int best_vert = -1;
+	bool got_vert = false;
+	size_t best_vert = 0;
 	double best_segang = 0;
-	for(int i=(start_idx+1)%npts; ; i=(i+1)%npts) {
+	for(size_t i=(start_idx+1)%npts; ; i=(i+1)%npts) {
 		if(i == start_idx) break;
-		vertex_t v1 = pts[i];
+		Vertex v1 = pts[i];
 		double segang = seg_ang(v0, v1);
 //printf("start_ang=%g*PI, seg_ang=%g*PI\n", start_ang/M_PI, segang/M_PI);
 		double angdiff = segang - start_ang;
@@ -78,39 +107,41 @@ static int find_next_convex(ring_t *ring, int start_idx, int limit_idx, double s
 			if(DEBUG) printf("test for seg crosses initial ray (%g*PI and %g*PI)\n", last_ad, angdiff);
 			if(fabs(last_ad-angdiff) > M_PI) {
 				if(DEBUG) printf("seg crosses initial ray (%g*PI and %g*PI)\n", last_ad, angdiff);
-				return -1;
+				return FindNextConvexRetval::ERROR();
 			}
 		}
 		last_ad = angdiff;
 		if(angdiff < min_angdiff) {
 			min_angdiff = angdiff;
+			got_vert = true;
 			best_vert = i;
 			best_segang = segang;
 		}
 		if(i == limit_idx) break;
 	}
-	if(best_vert < 0) {
-		return limit_idx;
+	if(!got_vert) {
+		return FindNextConvexRetval(limit_idx,
+			std::numeric_limits<double>::signaling_NaN());
 	} else if(min_angdiff >= M_PI) {
-		if(DEBUG) printf("point on wrong side of half-plane (ang=%g*PI) idx=%d\n", min_angdiff/M_PI, best_vert);
-		return -1;
+		if(DEBUG) printf("point on wrong side of half-plane (ang=%g*PI) idx=%zd\n", min_angdiff/M_PI, best_vert);
+		return FindNextConvexRetval::ERROR();
 	} else {
-		if(ang_out) *ang_out = best_segang;
-		return best_vert;
+		return FindNextConvexRetval(best_vert, best_segang);
 	}
 }
 
-static bool *find_chull(ring_t *ring) {
-	bool *keep = MYALLOC(bool, ring->npts);
-	for(int i=0; i<ring->npts; i++) keep[i] = false;
+static Keeps find_chull(const Ring &ring) {
+	Keeps keep(ring.pts.size(), false);
 
-	int start_idx = find_bottom_pt(ring);
+	size_t start_idx = find_bottom_pt(ring);
 	keep[start_idx] = true;
 	double ang = 0;
-	int idx = start_idx;
+	size_t idx = start_idx;
 	for(;;) {
-		idx = find_next_convex(ring, idx, start_idx, ang, &ang);
-		if(idx < 0) fatal_error("could not get convex hull");
+		FindNextConvexRetval r = find_next_convex(ring, idx, start_idx, ang);
+		if(r.error) fatal_error("could not get convex hull");
+		idx = r.idx;
+		ang = r.ang;
 		if(idx == start_idx) break;
 		keep[idx] = true;
 	}
@@ -118,14 +149,13 @@ static bool *find_chull(ring_t *ring) {
 	return keep;
 }
 
-static double subring_area(ring_t *ring, int from, int to) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static double subring_area(const Ring &ring, size_t from, size_t to) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
 
 	double accum = 0;
-	int i;
-	for(i=from; ; i=(i+1)%npts) {
-		int i2 = i==to ? from : (i+1)%npts;
+	for(size_t i=from; ; i=(i+1)%npts) {
+		size_t i2 = i==to ? from : (i+1)%npts;
 		double x0 = pts[i].x;
 		double y0 = pts[i].y;
 		double x1 = pts[i2].x;
@@ -142,83 +172,89 @@ static double subring_area(ring_t *ring, int from, int to) {
 	return accum / 2.0;
 }
 
-static int next_keep(int npts, bool *keep, int i) {
-	for(int j=(i+1)%npts; j!=i; j=(j+1)%npts) {
+static size_t next_keep(size_t npts, const Keeps &keep, size_t i) {
+	size_t i_plus1 = (i+1<npts) ? (i+1) : 0;
+	for(size_t j=i_plus1; j<npts; j++) {
+		if(keep[j]) return j;
+	}
+	for(size_t j=0; j<i; j++) {
 		if(keep[j]) return j;
 	}
 	return i;
 }
 
-static int prev_keep(int npts, bool *keep, int i) {
-	for(int j=(i+npts-1)%npts; j!=i; j=(j+npts-1)%npts) {
+static size_t prev_keep(size_t npts, const Keeps &keep, size_t i) {
+	for(size_t j=(i+npts-1)%npts; j!=i; j=(j+npts-1)%npts) {
 		if(keep[j]) return j;
 	}
 	return i;
 }
 
-static int reach_point(ring_t *ring, bool *keep, int from, int to, double ang) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static bool reach_point(const Ring &ring, Keeps &keep, size_t from, size_t to, double ang) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
 
-	if(DEBUG) printf("  reach %d and %d\n", from, to);
+	if(DEBUG) printf("  reach %zd and %zd\n", from, to);
 	if(DEBUG) printf("  %g,%g : %g,%g ang=%g*PI\n",
 		pts[from].x, pts[from].y, pts[to].x, pts[to].y, ang/M_PI);
 
-	int idx = from;
+	size_t idx = from;
 	for(;;) {
-		idx = find_next_convex(ring, idx, to, ang, &ang);
-		if(idx < 0) return -1;
+		FindNextConvexRetval r = find_next_convex(ring, idx, to, ang);
+		if(r.error) return 1;
+		idx = r.idx;
+		ang = r.ang;
 		keep[idx] = true;
 		if(idx == to) break;
 	}
-	for(int pk=from; pk!=to;) {
-		int nk = next_keep(npts, keep, pk);
+	for(size_t pk=from; pk!=to;) {
+		size_t nk = next_keep(npts, keep, pk);
 		if(DEBUG) printf("test seg %g,%g : %g,%g\n", pts[pk].x, pts[pk].y, pts[nk].x, pts[nk].y);
 
-		double min_x = MIN(pts[pk].x, pts[nk].x);
-		double max_x = MAX(pts[pk].x, pts[nk].x);
-		double min_y = MIN(pts[pk].y, pts[nk].y);
-		double max_y = MAX(pts[pk].y, pts[nk].y);
+		double min_x = std::min(pts[pk].x, pts[nk].x);
+		double max_x = std::max(pts[pk].x, pts[nk].x);
+		double min_y = std::min(pts[pk].y, pts[nk].y);
+		double max_y = std::max(pts[pk].y, pts[nk].y);
 
 		// FIXME - this is the slowest part
 		// FIXME - doesn't handle crossing across a vertex here or in dp.c
-		for(int i=0; i<npts; i++) {
-			int i2 = i+1<npts ? i+1 : 0;
-			vertex_t p1 = pts[i];
-			vertex_t p2 = pts[i2];
+		for(size_t i=0; i<npts; i++) {
+			size_t i2 = i+1<npts ? i+1 : 0;
+			Vertex p1 = pts[i];
+			Vertex p2 = pts[i2];
 			// the bbox test is copied here from line_intersects_line to save
 			// the time penalty of a function call for the common case of
 			// the bbox not matching
 			if(!(
-				max_x < MIN(p1.x, p2.x) ||
-				min_x > MAX(p1.x, p2.x) ||
-				max_y < MIN(p1.y, p2.y) ||
-				min_y > MAX(p1.y, p2.y) ||
+				max_x < std::min(p1.x, p2.x) ||
+				min_x > std::max(p1.x, p2.x) ||
+				max_y < std::min(p1.y, p2.y) ||
+				min_y > std::max(p1.y, p2.y) ||
 				i==pk || i==nk || i2==pk || i2==nk
 			)) {
 				if(line_intersects_line(pts[pk], pts[nk], p1, p2, 0)) {
 					if(DEBUG) printf("line intersects line\n");
-					return -1;
+					return 1;
 				}
 			}
 		}
-		for(int i=nk; ; ) {
-			int i2 = next_keep(npts, keep, i);
-			vertex_t p1 = pts[i];
-			vertex_t p2 = pts[i2];
+		for(size_t i=nk; ; ) {
+			size_t i2 = next_keep(npts, keep, i);
+			Vertex p1 = pts[i];
+			Vertex p2 = pts[i2];
 			// the bbox test is copied here from line_intersects_line to save
 			// the time penalty of a function call for the common case of
 			// the bbox not matching
 			if(!(
-				max_x < MIN(p1.x, p2.x) ||
-				min_x > MAX(p1.x, p2.x) ||
-				max_y < MIN(p1.y, p2.y) ||
-				min_y > MAX(p1.y, p2.y) ||
+				max_x < std::min(p1.x, p2.x) ||
+				min_x > std::max(p1.x, p2.x) ||
+				max_y < std::min(p1.y, p2.y) ||
+				min_y > std::max(p1.y, p2.y) ||
 				i==pk || i==nk || i2==pk || i2==nk
 			)) {
 				if(line_intersects_line(pts[pk], pts[nk], p1, p2, 0)) {
 					if(DEBUG) printf("line intersects line\n");
-					return -1;
+					return 1;
 				}
 			}
 			i = i2;
@@ -231,37 +267,37 @@ static int reach_point(ring_t *ring, bool *keep, int from, int to, double ang) {
 	return 0;
 }
 
-static int add_tiepoint(ring_t *ring, bool *keep, int mid) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static bool add_tiepoint(const Ring &ring, Keeps &keep, size_t mid) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
 
 	keep[mid] = true;
-	int left = prev_keep(npts, keep, mid);
-	int right = next_keep(npts, keep, mid);
-	if(DEBUG) printf("adding %d between %d and %d\n", mid, left, right);
+	size_t left = prev_keep(npts, keep, mid);
+	size_t right = next_keep(npts, keep, mid);
+	if(DEBUG) printf("adding %zd between %zd and %zd\n", mid, left, right);
 	if(DEBUG) printf("%g,%g : %g,%g : %g,%g\n",
 		pts[mid].x, pts[mid].y, pts[left].x, pts[left].y, pts[right].x, pts[right].y);
 
 	double ang = seg_ang(pts[left], pts[right]);
-	int error = reach_point(ring, keep, left, mid, ang);
-	if(error) return -1;
+	bool error = reach_point(ring, keep, left, mid, ang);
+	if(error) return 1;
 
-	int pk = prev_keep(npts, keep, mid);
+	size_t pk = prev_keep(npts, keep, mid);
 	if(pk == mid) fatal_error("pk == mid");
 	ang = seg_ang(pts[mid], pts[pk]);
 	error = reach_point(ring, keep, mid, right, ang);
-	if(error) return -1;
+	if(error) return 1;
 
 	return 0;
 }
 
 /*
-static int chord_crosses_arc(ring_t *ring, int c0, int c1, int from, int to) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static bool chord_crosses_arc(const Ring &ring, size_t c0, size_t c1, size_t from, size_t to) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
 
-	for(int i=from; i!=to; i=(i+1)%npts) {
-		int i2 = (i+1)%npts;
+	for(size_t i=from; i!=to; i=(i+1)%npts) {
+		size_t i2 = (i+1)%npts;
 		if(i==c0 || i==c1 || i2==c0 || i2==c1) continue;
 		if(line_intersects_line(pts[c0], pts[c1], pts[i], pts[i2], 0)) return 1;
 	}
@@ -270,7 +306,7 @@ static int chord_crosses_arc(ring_t *ring, int c0, int c1, int from, int to) {
 */
 
 // distance of p3 from line (p1,p2)
-static double dist_to_seg(vertex_t p1, vertex_t p2, vertex_t p3) {
+static double dist_to_seg(Vertex p1, Vertex p2, Vertex p3) {
 	double d21x = p2.x - p1.x; double d21y = p2.y - p1.y;
 	double d13x = p1.x - p3.x; double d13y = p1.y - p3.y;
 	// from http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html
@@ -278,13 +314,13 @@ static double dist_to_seg(vertex_t p1, vertex_t p2, vertex_t p3) {
 	return dist_from_line;
 }
 
-static int is_mostly_linear(ring_t *ring, int from, int to) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static bool is_mostly_linear(const Ring &ring, size_t from, size_t to) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
 
-	vertex_t p1 = pts[from];
-	vertex_t p2 = pts[to];
-	for(int i=(from+1)%npts; i!=to; i=(i+1)%npts) {
+	Vertex p1 = pts[from];
+	Vertex p2 = pts[to];
+	for(size_t i=(from+1)%npts; i!=to; i=(i+1)%npts) {
 		double dist = dist_to_seg(p1, p2, pts[i]);
 		if(dist > 1.0) return 0;
 	}
@@ -292,20 +328,20 @@ static int is_mostly_linear(ring_t *ring, int from, int to) {
 	return 1;
 }
 
-static int keep_linears(ring_t *ring, bool *keep_orig, int from, int to, bool *touchpts) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static bool keep_linears(const Ring &ring, Keeps &keep_orig, size_t from, size_t to, Keeps &touchpts) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
 
 	if(to == (from+1)%npts) return 0;
 
-	bool *keep_new = MYALLOC(bool, npts);
+	Keeps keep_new(npts);
 
 	double min_length = 20; // FIXME
 
-	for(int l_idx=from; l_idx!=to; l_idx=(l_idx+1)%npts) {
-		int longest = l_idx;
+	for(size_t l_idx=from; l_idx!=to; l_idx=(l_idx+1)%npts) {
+		size_t longest = l_idx;
 		double perim = 0;
-		for(int r_idx=(l_idx+1)%npts; ; r_idx=(r_idx+1)%npts) {
+		for(size_t r_idx=(l_idx+1)%npts; ; r_idx=(r_idx+1)%npts) {
 			perim += seg_len(pts[(r_idx+npts-1)%npts], pts[r_idx]);
 			if(perim > min_length && is_mostly_linear(ring, l_idx, r_idx)) {
 				longest = r_idx;
@@ -315,55 +351,53 @@ static int keep_linears(ring_t *ring, bool *keep_orig, int from, int to, bool *t
 			if(r_idx == to) break;
 		}
 		if(longest != l_idx) {
-			memcpy(keep_new, keep_orig, sizeof(bool) * npts);
-			int error = 0;
+			std::copy(keep_orig.begin(), keep_orig.end(), keep_new.begin());
+			bool error = 0;
 			if(l_idx != from) {
-				if(add_tiepoint(ring, keep_new, l_idx) < 0) error++;
+				if(add_tiepoint(ring, keep_new, l_idx)) error = 1;
 			}
 //if(error) fatal_error("oops"); // FIXME
 			if(longest != to) {
-				if(add_tiepoint(ring, keep_new, longest) < 0) error++;
+				if(add_tiepoint(ring, keep_new, longest)) error = 1;
 			}
 //if(error) fatal_error("oops"); // FIXME
 			if(!error) {
 				touchpts[l_idx] = true;
 				touchpts[longest] = true;
-				for(int i=l_idx; ; i=(i+1)%npts) {
+				for(size_t i=l_idx; ; i=(i+1)%npts) {
 					keep_new[i] = true;
 					if(i == longest) break;
 				}
-				memcpy(keep_orig, keep_new, sizeof(bool) * npts);
-				free(keep_new);
+				std::copy(keep_new.begin(), keep_new.end(), keep_orig.begin());
 				return 1;
 			}
 		}
 	}
-	free(keep_new);
 	return 0;
 }
 
-static int refine_seg(ring_t *ring, bool *keep_orig, int from, int to) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static std::pair<bool, size_t> refine_seg(const Ring &ring, Keeps &keep_orig, size_t from, size_t to) {
+	size_t npts = ring.pts.size();
+	const std::vector<Vertex> &pts = ring.pts;
 
 	double start_area = subring_area(ring, from, to);
 	double start_perim = seg_len(pts[from], pts[to]);
 
-	bool *keep_new = MYALLOC(bool, npts);
-	bool *keep_best = MYALLOC(bool, npts);
+	Keeps keep_new(npts);
+	Keeps keep_best(npts);
 
 	double best_improvement = 0;
-	int best_touchpt = -1;
+	size_t best_touchpt = 0;
 
-	for(int testpt=(from+1)%npts; testpt!=to; testpt=(testpt+1)%npts) {
-		memcpy(keep_new, keep_orig, sizeof(bool) * npts);
+	for(size_t testpt=(from+1)%npts; testpt!=to; testpt=(testpt+1)%npts) {
+		std::copy(keep_orig.begin(), keep_orig.end(), keep_new.begin());
 
-		if(add_tiepoint(ring, keep_new, testpt) < 0) continue;
+		if(add_tiepoint(ring, keep_new, testpt)) continue;
 
 		double left_area = 0;
 		double left_perim = 0;
-		for(int pk=from;;) {
-			int nk = next_keep(npts, keep_new, pk);
+		for(size_t pk=from;;) {
+			size_t nk = next_keep(npts, keep_new, pk);
 			left_area += subring_area(ring, pk, nk);
 			left_perim += seg_len(pts[pk], pts[nk]);
 			if(nk == testpt) break;
@@ -371,8 +405,8 @@ static int refine_seg(ring_t *ring, bool *keep_orig, int from, int to) {
 		}
 		double right_area = 0;
 		double right_perim = 0;
-		for(int pk=testpt;;) {
-			int nk = next_keep(npts, keep_new, pk);
+		for(size_t pk=testpt;;) {
+			size_t nk = next_keep(npts, keep_new, pk);
 			right_area += subring_area(ring, pk, nk);
 			right_perim += seg_len(pts[pk], pts[nk]);
 			if(nk == to) break;
@@ -403,46 +437,46 @@ static int refine_seg(ring_t *ring, bool *keep_orig, int from, int to) {
 
 		if(improvement > best_improvement) {
 			best_improvement = improvement;
-			memcpy(keep_best, keep_new, sizeof(bool) * npts);
+			std::copy(keep_new.begin(), keep_new.end(), keep_best.begin());
 			best_touchpt = testpt;
 		}
 	}
 	if(best_improvement) {
 		if(VERBOSE) {
 			printf("best_improvement = %g\n", best_improvement);
-			printf("tagged %d (%g,%g) as keep between %d and %d\n", best_touchpt,
+			printf("tagged %zd (%g,%g) as keep between %zd and %zd\n", best_touchpt,
 				pts[best_touchpt].x, pts[best_touchpt].y, from, to);
-			for(int i=0; i<npts; i++) {
+			for(size_t i=0; i<npts; i++) {
 				if(keep_best[i] && !keep_orig[i]) printf(
-					"  rubberband touches %d (%g, %g)\n", i, pts[i].x, pts[i].y);
+					"  rubberband touches %zd (%g, %g)\n", i, pts[i].x, pts[i].y);
 			}
 		}
 
-		memcpy(keep_orig, keep_best, sizeof(bool) * npts);
+		std::copy(keep_best.begin(), keep_best.end(), keep_orig.begin());
+		return std::pair<bool, size_t>(true, best_touchpt);
 	} else {
-		best_touchpt = -1;
+		return std::pair<bool, size_t>(false, 0);
 	}
-	free(keep_new);
-	free(keep_best);
-	return best_touchpt;
 }
 
-static void refine_ring(ring_t *ring, bool *keep, bool *touchpts) {
-	int npts = ring->npts;
-	for(int i=0; i<npts; i++) {
+static void refine_ring(const Ring &ring, Keeps &keep, Keeps &touchpts) {
+	size_t npts = ring.pts.size();
+	for(size_t i=0; i<npts; i++) {
 		if(!keep[i]) continue;
 		for(;;) {
 		// FIXME quick loop if j==i+1
-			int j = next_keep(npts, keep, i);
+			size_t j = next_keep(npts, keep, i);
 			double area = subring_area(ring, i, j);
-			if(VERBOSE) printf("area = %g, refining segment %d,%d\n", area, i, j);
+			if(VERBOSE) printf("area = %g, refining segment %zd,%zd\n", area, i, j);
 			if(area > 0) {
 if(VERBOSE) printf("do linear\n");
-				int did_linear = keep_linears(ring, keep, i, j, touchpts);
+				bool did_linear = keep_linears(ring, keep, i, j, touchpts);
 				if(did_linear) continue;
 if(VERBOSE) printf("do refine\n");
-				int touchpt = refine_seg(ring, keep, i, j);
-				if(touchpt >= 0) {
+				std::pair<bool, size_t> r = refine_seg(ring, keep, i, j);
+				bool got_touchpt = r.first;
+				size_t touchpt = r.second;
+				if(got_touchpt) {
 					touchpts[touchpt] = true;
 					continue;
 				}
@@ -452,62 +486,56 @@ if(VERBOSE) printf("do refine\n");
 	}
 }
 
-static ring_t pinch_ring_excursions(ring_t *ring) {
-	int npts = ring->npts;
-	vertex_t *pts = ring->pts;
+static Ring pinch_ring_excursions(const Ring &ring_in) {
+	Ring ring(ring_in);
+	size_t npts = ring.pts.size();
 
-	if(!ring_is_ccw(ring)) {
+	if(!ring.isCCW()) {
 		// reverse ring to make it CCW
-		for(int i=0; i<npts/2; i++) {
-			vertex_t tmp = pts[i];
-			pts[i] = pts[npts-1-i];
-			pts[npts-1-i] = tmp;
-		}
+		ring.reverse();
 	}
 
-	bool *keep = find_chull(ring);
-	bool *touchpts = MYALLOC(bool, ring->npts);
-	for(int i=0; i<ring->npts; i++) touchpts[i] = false;
+	const std::vector<Vertex> &pts = ring.pts;
+
+	Keeps keep = find_chull(ring);
+	Keeps touchpts(ring.pts.size());
+	for(size_t i=0; i<ring.pts.size(); i++) touchpts[i] = false;
 
 	refine_ring(ring, keep, touchpts);
 
-	int nkeep = 0;
-	for(int i=0; i<npts; i++) {
+	size_t nkeep = 0;
+	for(size_t i=0; i<npts; i++) {
 		if(keep[i]) nkeep++;
 	}
 
-	ring_t outring = *ring;
-	outring.npts = nkeep;
-	outring.pts = MYALLOC(vertex_t, outring.npts);
-	int oidx=0;
-	for(int i=0; i<npts; i++) {
-		if(keep[i]) outring.pts[oidx++] = pts[i];
+	Ring outring = ring.copyMetadata();
+	outring.pts.reserve(nkeep);
+	for(size_t i=0; i<npts; i++) {
+		if(keep[i]) outring.pts.push_back(pts[i]);
 	}
+	assert(outring.pts.size() == nkeep);
 
 //	if(dbuf && dbuf->mode == PLOT_PINCH) {
-//		debug_plot_ring(dbuf, &outring, 255, 0, 0);
-//		for(int i=0; i<npts; i++) {
+//		dbuf->debugPlotRing(&outring, 255, 0, 0);
+//		for(size_t i=0; i<npts; i++) {
 //			if(touchpts[i]) {
-//				vertex_t p = pts[i];
-//				plot_point_big(dbuf, p.x, p.y, 255, 255, 255);
+//				Vertex p = pts[i];
+//				dbuf->plotPointBig(p.x, p.y, 255, 255, 255);
 //			}
 //		}
 //	}
 
-	free(keep);
-	free(touchpts);
-
 	return outring;
 }
 
-static OGRGeometryH ring_to_ogrpoly(ring_t *r) {
+static OGRGeometryH ring_to_ogrpoly(const Ring &r) {
 	OGRGeometryH ogr = OGR_G_CreateGeometry(wkbPolygon);
 	OGR_G_AddGeometry(ogr, ring_to_ogr(r));
 	return ogr;
 }
 
 // rings must cross for this function
-static ring_t ring_ring_union(ring_t *r1, ring_t *r2) {
+static Ring ring_ring_union(const Ring &r1, const Ring &r2) {
 	OGRGeometryH og1 = ring_to_ogrpoly(r1);
 	OGRGeometryH og2 = ring_to_ogrpoly(r2);
 	OGRGeometryH og3 = OGR_G_Union(og1, og2);
@@ -515,7 +543,7 @@ static ring_t ring_ring_union(ring_t *r1, ring_t *r2) {
 	OGRwkbGeometryType type = OGR_G_GetGeometryType(og3);
 	if(type == wkbPolygon) {
 		// only take outer ring
-		ring_t r3 = ogr_to_ring(OGR_G_GetGeometryRef(og3, 0));
+		Ring r3 = ogr_to_ring(OGR_G_GetGeometryRef(og3, 0));
 		r3.is_hole = 0;
 		r3.parent_id = 0;
 		return r3;
@@ -524,51 +552,51 @@ static ring_t ring_ring_union(ring_t *r1, ring_t *r2) {
 	}
 }
 
-mpoly_t pinch_excursions2(mpoly_t *mp_in, report_image_t *dbuf) {
-	mpoly_t mp_out;
-	mp_out.num_rings = mp_in->num_rings;
-	mp_out.rings = MYALLOC(ring_t, mp_out.num_rings);
-	for(int r_idx=0; r_idx<mp_in->num_rings; r_idx++) {
-		// FIXME - put a test for this into usage()
-		if(mp_in->rings[r_idx].is_hole) fatal_error("pincher cannot be used on holes");
-		mp_out.rings[r_idx] = pinch_ring_excursions(&mp_in->rings[r_idx]);
+Mpoly pinch_excursions2(const Mpoly &mp_in, DebugPlot *dbuf) {
+	Mpoly mp_out;
+	mp_out.rings.resize(mp_in.rings.size());
+	for(size_t r_idx=0; r_idx<mp_in.rings.size(); r_idx++) {
+		// Some extra logic would be needed to support holes.  There is a test
+		// in gdal_trace_outline that ensures that the -no-donuts option is
+		// given.  This should have prevented holes.
+		if(mp_in.rings[r_idx].is_hole) fatal_error("pincher cannot be used on holes");
+		mp_out.rings[r_idx] = pinch_ring_excursions(mp_in.rings[r_idx]);
 	}
-	for(int r1_idx=0; r1_idx<mp_out.num_rings; r1_idx++) {
+	for(size_t r1_idx=0; r1_idx<mp_out.rings.size(); r1_idx++) {
 		REDO_R1:
-		if(r1_idx >= mp_out.num_rings) break;
+		if(r1_idx >= mp_out.rings.size()) break;
 
-		for(int r2_idx=r1_idx+1; r2_idx<mp_out.num_rings; r2_idx++) {
+		for(size_t r2_idx=r1_idx+1; r2_idx<mp_out.rings.size(); r2_idx++) {
 			REDO_R2:
-			if(r2_idx >= mp_out.num_rings) break;
+			if(r2_idx >= mp_out.rings.size()) break;
 
-			int rel = ring_ring_relation(&mp_out.rings[r1_idx], &mp_out.rings[r2_idx]);
-//printf("relation of %d and %d is %d\n", r1_idx, r2_idx, rel);
-			if(rel == RING_CONTAINS) {
-//printf("deleting %d\n", r2_idx);
+			RingRelation rel = ring_ring_relation(mp_out.rings[r1_idx], mp_out.rings[r2_idx]);
+//printf("relation of %zd and %zd is %zd\n", r1_idx, r2_idx, rel);
+			if(rel == RINGREL_CONTAINS) {
+//printf("deleting %zd\n", r2_idx);
 				//if(dbuf && dbuf->mode == PLOT_PINCH) {
-				//	debug_plot_ring(dbuf, &mp_out.rings[r2_idx], 0, 0, 128);
+				//	dbuf->debugPlotRing(mp_out.rings[r2_idx], 0, 0, 128);
 				//}
-				delete_ring_from_mpoly(&mp_out, r2_idx);
+				mp_out.deleteRing(r2_idx);
 				goto REDO_R2; // indexes shifted - reset loop
-			} else if(rel == RING_CONTAINED_BY) {
-//printf("deleting %d\n", r1_idx);
+			} else if(rel == RINGREL_CONTAINED_BY) {
+//printf("deleting %zd\n", r1_idx);
 				//if(dbuf && dbuf->mode == PLOT_PINCH) {
-				//	debug_plot_ring(dbuf, &mp_out.rings[r1_idx], 0, 0, 128);
+				//	dbuf->debugPlotRing(mp_out.rings[r1_idx], 0, 0, 128);
 				//}
-				delete_ring_from_mpoly(&mp_out, r1_idx);
+				mp_out.deleteRing(r1_idx);
 				goto REDO_R1; // indexes shifted - reset loop
-			} else if(rel == RING_CROSSES) {
-//printf("merging %d and %d\n", r1_idx, r2_idx);
-				ring_t r3 = ring_ring_union(&mp_out.rings[r1_idx], &mp_out.rings[r2_idx]);
+			} else if(rel == RINGREL_CROSSES) {
+//printf("merging %zd and %zd\n", r1_idx, r2_idx);
+				Ring r3 = ring_ring_union(mp_out.rings[r1_idx], mp_out.rings[r2_idx]);
 				//if(dbuf && dbuf->mode == PLOT_PINCH) {
-				//	debug_plot_ring(dbuf, &mp_out.rings[r1_idx], 0, 0, 128);
-				//	debug_plot_ring(dbuf, &mp_out.rings[r2_idx], 0, 0, 128);
+				//	dbuf->debugPlotRing(mp_out.rings[r1_idx], 0, 0, 128);
+				//	dbuf->debugPlotRing(mp_out.rings[r2_idx], 0, 0, 128);
 				//}
-				delete_ring_from_mpoly(&mp_out, r2_idx);
-				free_ring(&mp_out.rings[r1_idx]);
+				mp_out.deleteRing(r2_idx);
 				mp_out.rings[r1_idx] = r3;
 				//if(dbuf && dbuf->mode == PLOT_PINCH) {
-				//	debug_plot_ring(dbuf, &mp_out.rings[r1_idx], 255, 128, 0);
+				//	dbuf->debugPlotRing(mp_out.rings[r1_idx], 255, 128, 0);
 				//}
 				goto REDO_R1; // indexes shifted - reset loop
 			}
@@ -576,11 +604,13 @@ mpoly_t pinch_excursions2(mpoly_t *mp_in, report_image_t *dbuf) {
 	}
 
 	if(dbuf && dbuf->mode == PLOT_PINCH) {
-		for(int i=0; i<mp_out.num_rings; i++) {
-			debug_plot_ring(dbuf, &mp_out.rings[i], 255, 0, 0);
+		for(size_t i=0; i<mp_out.rings.size(); i++) {
+			dbuf->debugPlotRing(mp_out.rings[i], 255, 0, 0);
 		}
 	}
 
 	// FIXME - fix topology using functions from dp.c
 	return mp_out;
 }
+
+} // namespace dangdal
