@@ -26,41 +26,53 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <cassert>
+
+#include <boost/lexical_cast.hpp>
+#include <boost/numeric/conversion/cast.hpp>
+
 #include "common.h"
 #include "ndv.h"
 
-void compute_histogram(GDALRasterBandH *src_bands, int band_count, ndv_def_t *ndv_def, 
-	size_t w, size_t h, size_t **histograms, int input_range);
-void print_band_stats(int band_idx, size_t *histogram, int input_range, size_t npix);
-void get_scale_from_stddev(size_t *histogram, int input_range,
+using namespace dangdal;
+
+std::vector<std::vector<size_t> > compute_histogram(
+	std::vector<GDALRasterBandH> src_bands, const NdvDef &ndv_def, 
+	size_t w, size_t h, int input_range
+);
+void print_band_stats(size_t band_idx, const std::vector<size_t> &histogram, size_t npix);
+void get_scale_from_stddev(const std::vector<size_t> &histogram,
 	double dst_avg, double dst_stddev, double *scale_out, double *offset_out);
-void get_scale_from_percentile(size_t *histogram, int input_range, int output_range,
+void get_scale_from_percentile(const std::vector<size_t> &histogram, int output_range,
 	double from_percentile, double to_percentile, double *scale_out, double *offset_out);
-void invert_histogram_to_gaussian(size_t *histogram_in, double variance, 
-	int *table_out, int input_range, int output_range, double max_rel_freq);
+std::vector<size_t> invert_histogram_to_gaussian(const std::vector<size_t> &histogram_in, double variance, 
+	int output_range, double max_rel_freq);
 void copyGeoCode(GDALDatasetH dst_ds, GDALDatasetH src_ds);
 
-void usage(const char *cmdname) {
-	// FIXME - layout/grammar
-	printf("Usage: %s <options> src.tif dst.tif\n\n", cmdname);
-	print_ndv_usage();
+void usage(const std::string &cmdname) {
+	printf("Usage: %s <options> src.tif dst.tif\n\n", cmdname.c_str());
+	NdvDef::printUsage();
 	printf("\
-  -outndv <output_nodata_val>             Output no-data value\n\
+  -outndv <output_nodata_val>        Output no-data value\n\
 \n\
 Stretch mode:\n\
   -linear-stretch <target_avg> <target_stddev>      Linear stretch to a target range\n\
   -percentile-range <from: 0.0-1.0> <to: 0.0-1.0>   Linear stretch using a percentile range of input\n\
   -histeq <target_stddev>                           Histogram normalize to a target bell curve\n\
 \n\
-Input must be either 8-bit or 16-bit.  Output is 8-bit.\n\
+Input must be either 8-bit or 16-bit (unsigned).  Output is 8-bit.\n\
 ");
 	exit(1);
 }
 
 int main(int argc, char *argv[]) {
-	const char *src_fn = NULL;
-	const char *dst_fn = NULL;
-	const char *output_format = NULL;
+	const std::string cmdname = argv[0];
+	if(argc == 1) usage(cmdname);
+	std::vector<std::string> arg_list = argv_to_list(argc, argv);
+
+	std::string src_fn;
+	std::string dst_fn;
+	std::string output_format;
 
 	char mode_histeq = 0;
 	char mode_stddev = 0;
@@ -71,103 +83,110 @@ int main(int argc, char *argv[]) {
 	double to_percentile = -1;
 	int out_ndv = 0, set_out_ndv = 0;
 
-	ndv_def_t ndv_def = init_ndv_options(&argc, &argv);
+	NdvDef ndv_def = NdvDef(arg_list);
 
-	int argp = 1;
-	while(argp < argc) {
-		char *arg = argv[argp++];
+	size_t argp = 1;
+	while(argp < arg_list.size()) {
+		const std::string &arg = arg_list[argp++];
 		// FIXME - check duplicate values
 		if(arg[0] == '-') {
-			if(!strcmp(arg, "-of")) {
-				if(argp == argc) usage(argv[0]);
-				output_format = argv[argp++];
-			} else if(!strcmp(arg, "-linear-stretch")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				dst_avg = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
+			try {
+				if(arg == "-of") {
+					if(argp == arg_list.size()) usage(cmdname);
+					output_format = arg_list[argp++];
+				} else if(arg == "-linear-stretch") {
+					if(argp == arg_list.size()) usage(cmdname);
+					dst_avg = boost::lexical_cast<double>(arg_list[argp++]);
 
-				if(argp == argc) usage(argv[0]);
-				dst_stddev = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
+					if(argp == arg_list.size()) usage(cmdname);
+					dst_stddev = boost::lexical_cast<double>(arg_list[argp++]);
 
-				mode_stddev = 1;
-			} else if(!strcmp(arg, "-percentile-range")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				from_percentile = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
+					mode_stddev = 1;
+				} else if(arg == "-percentile-range") {
+					if(argp == arg_list.size()) usage(cmdname);
+					from_percentile = boost::lexical_cast<double>(arg_list[argp++]);
 
-				if(argp == argc) usage(argv[0]);
-				to_percentile = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
+					if(argp == arg_list.size()) usage(cmdname);
+					to_percentile = boost::lexical_cast<double>(arg_list[argp++]);
 
-				mode_percentile = 1;
-			} else if(!strcmp(arg, "-histeq")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				dst_stddev = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
+					mode_percentile = 1;
+				} else if(arg == "-histeq") {
+					if(argp == arg_list.size()) usage(cmdname);
+					dst_stddev = boost::lexical_cast<double>(arg_list[argp++]);
 
-				mode_histeq = 1;
-			} else if(!strcmp(arg, "-outndv")) {
- 				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				long ndv_long = strtol(argv[argp++], &endptr, 10);
-				out_ndv = (uint8_t)ndv_long;
-				if(ndv_long != (long)out_ndv) fatal_error("ndv must be in the range 0..255");
-				set_out_ndv++;
-				if(*endptr) usage(argv[0]);
-			} else usage(argv[0]);
+					mode_histeq = 1;
+				} else if(arg == "-outndv") {
+					if(argp == arg_list.size()) usage(cmdname);
+					int64_t ndv_long = boost::lexical_cast<int64_t>(arg_list[argp++]);
+					if(ndv_long < 0 || ndv_long > 255) fatal_error("ndv must be in the range 0..255");
+					out_ndv = boost::numeric_cast<uint8_t>(ndv_long);
+					set_out_ndv++;
+				} else {
+					usage(cmdname);
+				}
+			} catch(boost::bad_lexical_cast &e) {
+				fatal_error("cannot parse number given on command line");
+			} catch(boost::bad_numeric_cast &e) {
+				fatal_error("number given on command line out of range: %s", e.what());
+			}
 		} else {
-			if(!src_fn) {
+			if(src_fn.empty()) {
 				src_fn = arg;
-			} else if(!dst_fn) {
+			} else if(dst_fn.empty()) {
 				dst_fn = arg;
 			} else {
-				usage(argv[0]);
+				usage(cmdname);
 			}
 		}
 	}
 
-	if(!src_fn || !dst_fn) usage(argv[0]);
-	if(mode_percentile + mode_stddev + mode_histeq > 1) usage(argv[0]);
-	if(mode_stddev && (dst_avg < 0 || dst_stddev < 0)) usage(argv[0]);
+	if(src_fn.empty() || dst_fn.empty()) usage(cmdname);
+	if(mode_percentile + mode_stddev + mode_histeq > 1) usage(cmdname);
+	if(mode_stddev && (dst_avg < 0 || dst_stddev < 0)) usage(cmdname);
 	if(mode_percentile && !(
 		0 <= from_percentile && 
 		from_percentile < to_percentile &&
-		to_percentile <= 1)) usage(argv[0]);
+		to_percentile <= 1)) usage(cmdname);
 
-	if(!output_format) output_format = "GTiff";
+	if(output_format.empty()) output_format = "GTiff";
 
 	GDALAllRegister();
 
 	//////// open source ////////
 
-	GDALDatasetH src_ds = GDALOpen(src_fn, GA_ReadOnly);
+	GDALDatasetH src_ds = GDALOpen(src_fn.c_str(), GA_ReadOnly);
 	if(!src_ds) fatal_error("open failed");
 
 	size_t w = GDALGetRasterXSize(src_ds);
 	size_t h = GDALGetRasterYSize(src_ds);
 	if(!w || !h) fatal_error("missing width/height");
-	int band_count = GDALGetRasterCount(src_ds);
-	printf("Input size is %zd, %zd, %d\n", w, h, band_count);
+	size_t src_band_count = GDALGetRasterCount(src_ds);
+	printf("Input size is %zd, %zd, %zd\n", w, h, src_band_count);
 
-	if(!ndv_def.nranges) {
-		int *bandlist = MYALLOC(int, band_count);
-		for(int i=0; i<band_count; i++) bandlist[i] = i+1;
-		add_ndv_from_raster(&ndv_def, src_ds, band_count, bandlist);
+	std::vector<size_t> bandlist;
+	for(size_t i=0; i<src_band_count; i++) {
+		bandlist.push_back(i+1);
+	}
+	size_t dst_band_count = bandlist.size();
+
+	if(ndv_def.empty()) {
+		ndv_def = NdvDef(src_ds, bandlist);
 	}
 
-	bool use_ndv = (ndv_def.nranges > 0);
+	bool use_ndv = !ndv_def.empty();
 	if(use_ndv && !set_out_ndv) {
-		if(ndv_def.nranges == 1 && ndv_def.ranges[0].min[0] == ndv_def.ranges[0].max[0]) {
-			double v = ndv_def.ranges[0].min[0];
-			out_ndv = (uint8_t)v;
-			if((double)out_ndv == v) {
-				set_out_ndv++;
-			} else {
-				printf("Cannot use %g as an NDV value because it cannot be cast to an 8-bit number.\n", v);
+		if(ndv_def.slabs.size() == 1) {
+			const NdvSlab &slab = ndv_def.slabs[0];
+			assert(slab.range_by_band.size());
+			const NdvInterval &range = slab.range_by_band[0];
+			if(range.first == range.second) {
+				double v = range.first;
+				out_ndv = (uint8_t)v;
+				if((double)out_ndv == v) {
+					set_out_ndv++;
+				} else {
+					printf("Cannot use %g as an NDV value because it cannot be cast to an 8-bit number.\n", v);
+				}
 			}
 		}
 		if(!set_out_ndv) {
@@ -179,22 +198,22 @@ int main(int argc, char *argv[]) {
 
 	//////// open output ////////
 
-	printf("Output size is %zd x %zd x %d\n", w, h, band_count);
+	printf("Output size is %zd x %zd x %zd\n", w, h, dst_band_count);
 
-	GDALDriverH dst_driver = GDALGetDriverByName(output_format);
-	if(!dst_driver) fatal_error("unrecognized output format (%s)", output_format);
-	GDALDatasetH dst_ds = GDALCreate(dst_driver, dst_fn, w, h, band_count, GDT_Byte, NULL);
+	GDALDriverH dst_driver = GDALGetDriverByName(output_format.c_str());
+	if(!dst_driver) fatal_error("unrecognized output format (%s)", output_format.c_str());
+	GDALDatasetH dst_ds = GDALCreate(dst_driver, dst_fn.c_str(), w, h, dst_band_count, GDT_Byte, NULL);
 	if(!dst_ds) fatal_error("couldn't create output");
 	copyGeoCode(dst_ds, src_ds);
 
 	//////// open bands ////////
 
-	GDALRasterBandH *src_bands = MYALLOC(GDALRasterBandH, band_count);
-	GDALRasterBandH *dst_bands = MYALLOC(GDALRasterBandH, band_count);
+	std::vector<GDALRasterBandH> src_bands;
+	std::vector<GDALRasterBandH> dst_bands;
 
-	for(int band_idx=0; band_idx<band_count; band_idx++) {
-		src_bands[band_idx] = GDALGetRasterBand(src_ds, band_idx+1);
-		dst_bands[band_idx] = GDALGetRasterBand(dst_ds, band_idx+1);
+	for(size_t band_idx=0; band_idx<dst_band_count; band_idx++) {
+		src_bands.push_back(GDALGetRasterBand(src_ds, bandlist[band_idx]));
+		dst_bands.push_back(GDALGetRasterBand(dst_ds, band_idx+1));
 	}
 
 	//////// compute lookup table ////////
@@ -204,52 +223,47 @@ int main(int argc, char *argv[]) {
 
 	int mode_noeq = !(mode_percentile || mode_stddev || mode_histeq);
 
-	size_t **histograms = NULL;
+	std::vector<std::vector<size_t> > histograms;
 	if(!mode_noeq) {
-		histograms = MYALLOC(size_t *, band_count);
-		for(int band_idx=0; band_idx<band_count; band_idx++) {
-			histograms[band_idx] = MYALLOC(size_t, input_range);
-		}
-		compute_histogram(src_bands, band_count, &ndv_def, w, h, histograms, input_range);
+		histograms = compute_histogram(src_bands, ndv_def, w, h, input_range);
 		printf("\n");
 	}
 
-	int **xform_table = MYALLOC(int *, band_count);
-
-	for(int band_idx=0; band_idx<band_count; band_idx++) {
-		xform_table[band_idx] = MYALLOC(int, input_range);
+	std::vector<std::vector<size_t> > xform_table(dst_band_count);
 
+	for(size_t band_idx=0; band_idx<dst_band_count; band_idx++) {
 		if(mode_noeq) {
+			xform_table[band_idx].resize(input_range);
 			for(int i=0; i<input_range; i++) xform_table[band_idx][i] = i;
 		} else {
-			print_band_stats(band_idx, histograms[band_idx], input_range, w*h);
+			print_band_stats(band_idx, histograms[band_idx], w*h);
 			if(mode_stddev || mode_percentile) {
 				double scale, offset;
 				if(mode_stddev) {
-					get_scale_from_stddev(histograms[band_idx], input_range,
+					get_scale_from_stddev(histograms[band_idx],
 						dst_avg, dst_stddev, &scale, &offset);
 				} else if(mode_percentile) {
-					get_scale_from_percentile(histograms[band_idx], input_range, output_range,
+					get_scale_from_percentile(histograms[band_idx], output_range,
 						from_percentile, to_percentile, &scale, &offset);
 				}
 				printf("  scale=%f, offset=%f, src_range=[%f, %f]\n",
 					scale, offset, -offset/scale, ((double)(output_range-1)-offset)/scale);
+				xform_table[band_idx].resize(input_range);
 				for(int i=0; i<input_range; i++) {
 					xform_table[band_idx][i] = (int)( (double)i * scale + offset );
 				}
 			} else if(mode_histeq) {
-				invert_histogram_to_gaussian(histograms[band_idx], dst_stddev,
-					xform_table[band_idx], input_range, output_range, 5.0);
+				xform_table[band_idx] = invert_histogram_to_gaussian(
+					histograms[band_idx], dst_stddev, output_range, 5.0);
 			} else {
 				fatal_error("unrecognized mode");
 			}
-			free(histograms[band_idx]);
 		}
 	}
-	free(histograms);
+	histograms.clear(); // free up memory
 
 	// clipping
-	for(int band_idx=0; band_idx<band_count; band_idx++) {
+	for(size_t band_idx=0; band_idx<dst_band_count; band_idx++) {
 		for(int i=0; i<input_range; i++) {
 			int v = xform_table[band_idx][i];
 			if(v < 0) v = 0;
@@ -260,7 +274,7 @@ int main(int argc, char *argv[]) {
 
 	// avoid ndv in output for good pixels
 	if(use_ndv) {
-		for(int band_idx=0; band_idx<band_count; band_idx++) {
+		for(size_t band_idx=0; band_idx<dst_band_count; band_idx++) {
 			for(int i=0; i<input_range; i++) {
 				int v = xform_table[band_idx][i];
 				if(v == out_ndv) {
@@ -282,15 +296,15 @@ int main(int argc, char *argv[]) {
 	size_t blocksize_y = blocksize_y_int;
 	size_t block_len = blocksize_x*blocksize_y;
 
-	GUInt16 **buf_in = MYALLOC(GUInt16 *, band_count);
-	uint8_t **buf_out = MYALLOC(uint8_t *, band_count);
-	for(int band_idx=0; band_idx<band_count; band_idx++) {
-		buf_in[band_idx] = MYALLOC(GUInt16, block_len);
-		buf_out[band_idx] = MYALLOC(uint8_t, block_len);
+	std::vector<std::vector<GUInt16> > buf_in(dst_band_count);
+	std::vector<std::vector<uint8_t> > buf_out(dst_band_count);
+	for(size_t band_idx=0; band_idx<dst_band_count; band_idx++) {
+		buf_in[band_idx].resize(block_len);
+		buf_out[band_idx].resize(block_len);
 	}
-	double *buf_in_dbl = MYALLOC(double, block_len);
-	uint8_t *ndv_mask = MYALLOC(uint8_t, block_len);
-	uint8_t *band_mask = MYALLOC(uint8_t, block_len);
+	std::vector<double> buf_in_dbl(block_len);
+	std::vector<uint8_t> ndv_mask(block_len);
+	std::vector<uint8_t> band_mask(block_len);
 
 	for(size_t boff_y=0; boff_y<h; boff_y+=blocksize_y) {
 		size_t bsize_y = blocksize_y;
@@ -307,26 +321,26 @@ int main(int argc, char *argv[]) {
 				((double)w * (double)h);
 			GDALTermProgress(progress, NULL, NULL);
 
-			for(int band_idx=0; band_idx<band_count; band_idx++) {
+			for(size_t band_idx=0; band_idx<dst_band_count; band_idx++) {
 				GDALRasterIO(src_bands[band_idx], GF_Read, boff_x, boff_y, bsize_x, bsize_y, 
-					buf_in[band_idx], bsize_x, bsize_y, GDT_UInt16, 0, 0);
+					&buf_in[band_idx][0], bsize_x, bsize_y, GDT_UInt16, 0, 0);
 
 				for(size_t i=0; i<block_len; i++) {
 					buf_in_dbl[i] = buf_in[band_idx][i];
 				}
 				if(band_idx == 0) {
-					array_check_ndv(&ndv_def, band_idx, buf_in_dbl, NULL, ndv_mask, block_len);
+					ndv_def.arrayCheckNdv(band_idx, &buf_in_dbl[0], &ndv_mask[0], block_len);
 				} else {
-					array_check_ndv(&ndv_def, band_idx, buf_in_dbl, NULL, band_mask, block_len);
-					aggregate_ndv_mask(&ndv_def, ndv_mask, band_mask, block_len);
+					ndv_def.arrayCheckNdv(band_idx, &buf_in_dbl[0], &band_mask[0], block_len);
+					ndv_def.aggregateMask(&ndv_mask[0], &band_mask[0], block_len);
 				}
 			}
 
-			for(int band_idx=0; band_idx<band_count; band_idx++) {
-				int *xform = xform_table[band_idx];
-				GUInt16 *p_in = buf_in[band_idx];
-				uint8_t *p_out = buf_out[band_idx];
-				uint8_t *p_ndv = ndv_mask;
+			for(size_t band_idx=0; band_idx<dst_band_count; band_idx++) {
+				size_t *xform = &xform_table[band_idx][0];
+				GUInt16 *p_in = &buf_in[band_idx][0];
+				uint8_t *p_out = &buf_out[band_idx][0];
+				uint8_t *p_ndv = &ndv_mask[0];
 
 				for(size_t i=0; i<block_len; i++) {
 					if(*p_ndv) {
@@ -338,7 +352,7 @@ int main(int argc, char *argv[]) {
 				}
 
 				GDALRasterIO(dst_bands[band_idx], GF_Write, boff_x, boff_y, bsize_x, bsize_y, 
-					buf_out[band_idx], bsize_x, bsize_y, GDT_Byte, 0, 0);
+					&buf_out[band_idx][0], bsize_x, bsize_y, GDT_Byte, 0, 0);
 			} // band
 		} // block x
 	} // block y
@@ -351,14 +365,17 @@ int main(int argc, char *argv[]) {
 	return 0;
 }
 
-void compute_histogram(
-	GDALRasterBandH *src_bands, int band_count, ndv_def_t *ndv_def, 
-	size_t w, size_t h, size_t **histograms, int input_range
+std::vector<std::vector<size_t> > compute_histogram(
+	std::vector<GDALRasterBandH> src_bands, const NdvDef &ndv_def, 
+	size_t w, size_t h, int input_range
 ) {
 	printf("\nComputing histogram...\n");
 
-	for(int band_idx=0; band_idx<band_count; band_idx++) {
-		for(int i=0; i<input_range; i++) histograms[band_idx][i] = 0;
+	size_t band_count = src_bands.size();
+	std::vector<std::vector<size_t> > histograms(band_count);
+
+	for(size_t band_idx=0; band_idx<band_count; band_idx++) {
+		histograms[band_idx].assign(input_range, 0);
 	}
 
 	int blocksize_x_int, blocksize_y_int;
@@ -367,13 +384,13 @@ void compute_histogram(
 	size_t blocksize_y = blocksize_y_int;
 	size_t block_len = blocksize_x*blocksize_y;
 
-	GUInt16 **buf_in = MYALLOC(GUInt16 *, band_count);
-	for(int band_idx=0; band_idx<band_count; band_idx++) {
-		buf_in[band_idx] = MYALLOC(GUInt16, block_len);
+	std::vector<std::vector<GUInt16> > buf_in(band_count);
+	for(size_t band_idx=0; band_idx<band_count; band_idx++) {
+		buf_in[band_idx].resize(block_len);
 	}
-	double *buf_in_dbl = MYALLOC(double, block_len);
-	uint8_t *ndv_mask = MYALLOC(uint8_t, block_len);
-	uint8_t *band_mask = MYALLOC(uint8_t, block_len);
+	std::vector<double> buf_in_dbl(block_len);
+	std::vector<uint8_t> ndv_mask(block_len);
+	std::vector<uint8_t> band_mask(block_len);
 
 	for(size_t boff_y=0; boff_y<h; boff_y+=blocksize_y) {
 		size_t bsize_y = blocksize_y;
@@ -390,23 +407,23 @@ void compute_histogram(
 				((double)w * (double)h);
 			GDALTermProgress(progress, NULL, NULL);
 
-			for(int band_idx=0; band_idx<band_count; band_idx++) {
+			for(size_t band_idx=0; band_idx<band_count; band_idx++) {
 				GDALRasterIO(src_bands[band_idx], GF_Read, boff_x, boff_y, bsize_x, bsize_y, 
-					buf_in[band_idx], bsize_x, bsize_y, GDT_UInt16, 0, 0);
+					&buf_in[band_idx][0], bsize_x, bsize_y, GDT_UInt16, 0, 0);
 
 				for(size_t i=0; i<block_len; i++) {
 					buf_in_dbl[i] = buf_in[band_idx][i];
 				}
 				if(band_idx == 0) {
-					array_check_ndv(ndv_def, band_idx, buf_in_dbl, NULL, ndv_mask, block_len);
+					ndv_def.arrayCheckNdv(band_idx, &buf_in_dbl[0], &ndv_mask[0], block_len);
 				} else {
-					array_check_ndv(ndv_def, band_idx, buf_in_dbl, NULL, band_mask, block_len);
-					aggregate_ndv_mask(ndv_def, ndv_mask, band_mask, block_len);
+					ndv_def.arrayCheckNdv(band_idx, &buf_in_dbl[0], &band_mask[0], block_len);
+					ndv_def.aggregateMask(&ndv_mask[0], &band_mask[0], block_len);
 				}
 			}
-			for(int band_idx=0; band_idx<band_count; band_idx++) {
-				GUInt16 *p = buf_in[band_idx];
-				size_t *hg = histograms[band_idx];
+			for(size_t band_idx=0; band_idx<band_count; band_idx++) {
+				GUInt16 *p = &buf_in[band_idx][0];
+				size_t *hg = &histograms[band_idx][0];
 				
 				for(size_t i=0; i<block_len; i++) {
 					if(!ndv_mask[i]) hg[p[i]]++;
@@ -416,43 +433,37 @@ void compute_histogram(
 	}
 	GDALTermProgress(1, NULL, NULL);
 
-	for(int band_idx=0; band_idx<band_count; band_idx++) {
-		free(buf_in[band_idx]);
-	}
-	free(buf_in);
-	free(buf_in_dbl);
-	free(ndv_mask);
-	free(band_mask);
+	return histograms;
 }
 
-void print_band_stats(int band_idx, size_t *histogram, int input_range, size_t npix) {
+void print_band_stats(size_t band_idx, const std::vector<size_t> &histogram, size_t npix) {
 	size_t ngood=0;
-	int min=0, max=0;
-	for(int i=0; i<input_range; i++) {
+	size_t min=0, max=0;
+	for(size_t i=0; i<histogram.size(); i++) {
 		if(histogram[i]) {
 			if(!ngood || i < min) min = i;
 			if(!ngood || i > max) max = i;
 			ngood += histogram[i];
 		}
 	}
-	printf("Band %d:\n  valid_pixels=%zd, ndv_pixels=%zd, min=%d, max=%d\n", 
+	printf("Band %zd:\n  valid_pixels=%zd, ndv_pixels=%zd, min=%zd, max=%zd\n", 
 		band_idx, ngood, npix-ngood, min, max);
 }
 
 void get_scale_from_stddev(
-	size_t *histogram, int input_range,
+	const std::vector<size_t> &histogram,
 	double dst_avg, double dst_stddev,
 	double *scale_out, double *offset_out
 ) {
 	size_t num_pixels = 0;
 	double pixel_total = 0;
-	for(int i=0; i<input_range; i++) {
+	for(size_t i=0; i<histogram.size(); i++) {
 		num_pixels += histogram[i];
 		pixel_total += i*histogram[i];
 	}
 	double src_avg = pixel_total / (double)num_pixels;
 	double error_total = 0;
-	for(int i=0; i<input_range; i++) {
+	for(size_t i=0; i<histogram.size(); i++) {
 		double diff = (double)i - src_avg;
 		error_total += diff*diff*(double)histogram[i];
 	}
@@ -465,12 +476,12 @@ void get_scale_from_stddev(
 }
 
 void get_scale_from_percentile(
-	size_t *histogram, int input_range, int output_range,
+	const std::vector<size_t> &histogram, int output_range,
 	double from_percentile, double to_percentile,
 	double *scale_out, double *offset_out
 ) {
 	size_t num_pixels = 0;
-	for(int i=0; i<input_range; i++) num_pixels += histogram[i];
+	for(size_t i=0; i<histogram.size(); i++) num_pixels += histogram[i];
 
 	size_t start_count = (size_t)(num_pixels * from_percentile);
 	size_t end_count = (size_t)(num_pixels * to_percentile);
@@ -478,21 +489,21 @@ void get_scale_from_percentile(
 	size_t cnt = 0;
 	int from_val = -1;
 	int to_val = -1;
-	for(int i=0; i<input_range; i++) {
+	for(size_t i=0; i<histogram.size(); i++) {
 		if(cnt <= start_count) from_val = i;
 		cnt += histogram[i];
 		if(cnt <= end_count) to_val = i; 
 		else break;
 	}
 	if(from_val<0 || to_val<0) fatal_error("impossible: could not find window");
-	if(from_val == to_val) { from_val=0; to_val=input_range-1; } // FIXME
+	if(from_val == to_val) { from_val=0; to_val=histogram.size()-1; } // FIXME
 
 	*scale_out = (double)(output_range-1) / (double)(to_val-from_val);
 	*offset_out = -(double)from_val * (*scale_out);
 }
 
-double *gen_gaussian(double variance, int bin_count) {
-	double *arr = MYALLOC(double, bin_count);
+std::vector<double> gen_gaussian(double variance, int bin_count) {
+	std::vector<double> arr(bin_count);
 	double total = 0;
 	for(int i=0; i<bin_count; i++) {
 		if(variance == 0.0) {
@@ -513,14 +524,18 @@ double *gen_gaussian(double variance, int bin_count) {
 	return arr;
 }
 
-void invert_histogram(size_t *src_h, double *dst_h, int *out_h, 
-int input_range, int output_range, double max_rel_freq) {
-	int pixel_count = 0;
-	for(int i=0; i<input_range; i++) pixel_count += src_h[i];
+std::vector<size_t> invert_histogram(
+	const std::vector<size_t> &src_h_in,
+	const std::vector<double> dst_h,
+	size_t output_range, double max_rel_freq
+) {
+	std::vector<size_t> src_h(src_h_in);
+	size_t pixel_count = 0;
+	for(size_t i=0; i<src_h.size(); i++) pixel_count += src_h[i];
 
 	if(max_rel_freq) {
-		size_t bin_max = size_t(max_rel_freq * pixel_count / input_range);
-		for(int i=0; i<input_range; i++) {
+		size_t bin_max = size_t(max_rel_freq * pixel_count / src_h.size());
+		for(size_t i=0; i<src_h.size(); i++) {
 			if(src_h[i] > bin_max) {
 				pixel_count -= src_h[i] - bin_max;
 				src_h[i] = bin_max;
@@ -528,23 +543,27 @@ int input_range, int output_range, double max_rel_freq) {
 		}
 	}
 
+	std::vector<size_t> out_h(src_h.size());
 	double src_total = 0;
 	double dst_total = 0;
-	int j = 0;
-	for(int i=0; i<input_range; i++) {
+	size_t j = 0;
+	for(size_t i=0; i<src_h.size(); i++) {
 		out_h[i] = j;
 		src_total += src_h[i];
 		while(j<output_range && dst_total < src_total) {
 			dst_total += dst_h[j++] * (double)pixel_count;
 		}
 	}
+
+	return out_h;
 }
 
-void invert_histogram_to_gaussian(size_t *histogram_in, double variance, 
-int *table_out, int input_range, int output_range, double max_rel_freq) {
-	double *gaussian = gen_gaussian(variance, output_range);
-	invert_histogram(histogram_in, gaussian, table_out, input_range, output_range, max_rel_freq);
-	free(gaussian);
+std::vector<size_t> invert_histogram_to_gaussian(
+	const std::vector<size_t> &histogram_in, double variance, 
+	int output_range, double max_rel_freq
+) {
+	std::vector<double> gaussian = gen_gaussian(variance, output_range);
+	return invert_histogram(histogram_in, gaussian, output_range, max_rel_freq);
 }
 
 void copyGeoCode(GDALDatasetH dst_ds, GDALDatasetH src_ds) {
