@@ -26,46 +26,40 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <cassert>
+
+#include <boost/lexical_cast.hpp>
+
 #include "common.h"
 #include "georef.h"
 #include "ndv.h"
-#include "default_palette.h"
+#include "palette.h"
+
+using namespace dangdal;
 
 // these are global so that they can be printed by the usage() function
 double default_slope_exageration = 2.0;
 double default_lightvec[] = { 0, 1, 1.5 };
 double default_shade_params[] = { 0, 1, .5, 10 };
 
-#define SHADE_TABLE_SIZE 500
-#define SHADE_TABLE_SCALE 100.0
-#define EARTH_RADIUS 6370997.0
+static const int SHADE_TABLE_SIZE = 500;
+static const double SHADE_TABLE_SCALE = 100.0;
+static const double EARTH_RADIUS = 6370997.0;
 
 void scale_values(double *vals, size_t w, double scale, double offset);
 
-typedef struct {
-	uint8_t nan_red, nan_green, nan_blue;
-	int num_vals;
-	double *vals;
-	uint8_t *reds, *greens, *blues;
-} palette_t;
-
-palette_t *read_palette_file(const char *fn);
-palette_t *read_default_palette();
-void get_nan_color(uint8_t *buf, palette_t *pal);
-void get_palette_color(uint8_t *buf, double val, palette_t *pal);
-
 void compute_tierow_invaffine(
-	georef_t *georef,
+	const GeoRef &georef,
 	int num_cols, int row, int grid_spacing,
 	double *invaffine_tierow
 );
 
-void usage(const char *cmdname) {
-	printf("Usage: %s <options> src_dataset dst_dataset\n\n", cmdname);
+void usage(const std::string &cmdname) {
+	printf("Usage: %s <options> src_dataset dst_dataset\n\n", cmdname.c_str());
 	
-	print_georef_usage();
+	GeoOpts::printUsage();
 	printf("\n");
-	print_ndv_usage();
+	NdvDef::printUsage();
 	printf("\n");
 	printf("Input/Output:\n");
 	printf("  -b input_band_id\n");
@@ -98,18 +92,22 @@ void usage(const char *cmdname) {
 }
 
 int main(int argc, char *argv[]) {
+	const std::string cmdname = argv[0];
+	if(argc == 1) usage(cmdname);
+	std::vector<std::string> arg_list = argv_to_list(argc, argv);
+
 	double slope_exageration = default_slope_exageration;
 	double lightvec[3];
-	memcpy(lightvec, default_lightvec, 3*sizeof(double));
+	for(int i=0; i<3; i++) lightvec[i] = default_lightvec[i];
 	double shade_params[4];
-	memcpy(shade_params, default_shade_params, 4*sizeof(double));
+	for(int i=0; i<4; i++) shade_params[i] = default_shade_params[i];
 
-	const char *src_fn = NULL;
-	const char *tex_fn = NULL;
-	const char *dst_fn = NULL;
-	const char *palette_fn = NULL;
+	std::string src_fn;
+	std::string tex_fn;
+	std::string dst_fn;
+	std::string palette_fn;
 	bool use_default_palette = 0;
-	const char *output_format = NULL;
+	std::string output_format;
 	int grid_spacing = 20; // could be configurable...
 	int band_id = 1;
 	double src_offset = 0;
@@ -117,70 +115,64 @@ int main(int argc, char *argv[]) {
 	bool data24bit = 0;
 	bool alpha_overlay = 0;
 
-	geo_opts_t geo_opts = init_geo_options(&argc, &argv);
-	ndv_def_t ndv_def = init_ndv_options(&argc, &argv);
+	GeoOpts geo_opts = GeoOpts(arg_list);
+	NdvDef ndv_def = NdvDef(arg_list);
 
-	int argp = 1;
-	while(argp < argc) {
-		char *arg = argv[argp++];
+	size_t argp = 1;
+	while(argp < arg_list.size()) {
+		const std::string &arg = arg_list[argp++];
 		// FIXME - check duplicate values
 		if(arg[0] == '-') {
-			if(!strcmp(arg, "-b")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				band_id = strtol(argv[argp++], &endptr, 10);
-				if(*endptr) usage(argv[0]);
-			} else if(!strcmp(arg, "-palette")) {
-				if(argp == argc) usage(argv[0]);
-				palette_fn = argv[argp++];
-				if(!strcmp(palette_fn, "data24bit")) {
-					data24bit = 1;
-					palette_fn = NULL;
+			try {
+				if(arg == "-b") {
+					if(argp == arg_list.size()) usage(cmdname);
+					band_id = boost::lexical_cast<int>(arg_list[argp++]);
+				} else if(arg == "-palette") {
+					if(argp == arg_list.size()) usage(cmdname);
+					palette_fn = arg_list[argp++];
+					if(palette_fn == "data24bit") {
+						data24bit = 1;
+						palette_fn.erase();
+					}
+				} else if(arg == "-default-palette") {
+					use_default_palette = 1;
+				} else if(arg == "-texture") {
+					if(argp == arg_list.size()) usage(cmdname);
+					tex_fn = arg_list[argp++];
+				} else if(arg == "-alpha-overlay") {
+					alpha_overlay = 1;
+				} else if(arg == "-of") {
+					if(argp == arg_list.size()) usage(cmdname);
+					output_format = arg_list[argp++];
+				} else if(arg == "-exag") {
+					if(argp == arg_list.size()) usage(cmdname);
+					slope_exageration = boost::lexical_cast<double>(arg_list[argp++]);
+				} else if(arg == "-lightvec") {
+					for(int i=0; i<3; i++) {
+						if(argp == arg_list.size()) usage(cmdname);
+						lightvec[i] = boost::lexical_cast<double>(arg_list[argp++]);
+					}
+				} else if(arg == "-shade") {
+					for(int i=0; i<4; i++) {
+						if(argp == arg_list.size()) usage(cmdname);
+						shade_params[i] = boost::lexical_cast<double>(arg_list[argp++]);
+					}
+				} else if(arg == "-offset") {
+					if(argp == arg_list.size()) usage(cmdname);
+					src_offset = boost::lexical_cast<double>(arg_list[argp++]);
+				} else if(arg == "-scale") {
+					if(argp == arg_list.size()) usage(cmdname);
+					src_scale = boost::lexical_cast<double>(arg_list[argp++]);
+				} else {
+					usage(cmdname);
 				}
-			} else if(!strcmp(arg, "-default-palette")) {
-				use_default_palette = 1;
-			} else if(!strcmp(arg, "-texture")) {
-				if(argp == argc) usage(argv[0]);
-				tex_fn = argv[argp++];
-			} else if(!strcmp(arg, "-alpha-overlay")) {
-				alpha_overlay = 1;
-			} else if(!strcmp(arg, "-of")) {
-				if(argp == argc) usage(argv[0]);
-				output_format = argv[argp++];
-			} else if(!strcmp(arg, "-exag")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				slope_exageration = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
-			} else if(!strcmp(arg, "-lightvec")) {
-				for(int i=0; i<3; i++) {
-					if(argp == argc) usage(argv[0]);
-					char *endptr;
-					lightvec[i] = strtod(argv[argp++], &endptr);
-					if(*endptr) usage(argv[0]);
-				}
-			} else if(!strcmp(arg, "-shade")) {
-				for(int i=0; i<4; i++) {
-					if(argp == argc) usage(argv[0]);
-					char *endptr;
-					shade_params[i] = strtod(argv[argp++], &endptr);
-					if(*endptr) usage(argv[0]);
-				}
-			} else if(!strcmp(arg, "-offset")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				src_offset = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
-			} else if(!strcmp(arg, "-scale")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				src_scale = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
-			} else usage(argv[0]);
+			} catch(boost::bad_lexical_cast &e) {
+				fatal_error("cannot parse number given on command line");
+			}
 		} else {
-			if(src_fn && dst_fn) {
-				usage(argv[0]);
-			} else if(src_fn) {
+			if(src_fn.size() && dst_fn.size()) {
+				usage(cmdname);
+			} else if(src_fn.size()) {
 				dst_fn = arg;
 			} else {
 				src_fn = arg;
@@ -188,14 +180,14 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	if(!src_fn || !dst_fn) usage(argv[0]);
+	if(src_fn.empty() || dst_fn.empty()) usage(cmdname);
 
-	if(!output_format) output_format = "GTiff";
+	if(output_format.empty()) output_format = "GTiff";
 
 	int num_output_modes = 
 		(data24bit ? 1 : 0) +
 		(use_default_palette ? 1 : 0) +
-		(palette_fn ? 1 : 0) +
+		(palette_fn.size() ? 1 : 0) +
 		(alpha_overlay ? 1 : 0);
 	if(num_output_modes > 1) fatal_error("you can only use one of the -palette, -default-palette, -texture, and -alpha-overlay options");
 
@@ -205,7 +197,8 @@ int main(int argc, char *argv[]) {
 
 	//////// open palette ////////
 
-	palette_t *palette = NULL;
+	bool use_palette = false;
+	Palette palette;
 	bool do_shade;
 	int out_numbands;
 
@@ -213,12 +206,13 @@ int main(int argc, char *argv[]) {
 		do_shade = 0;
 		out_numbands = 3;
 	} else if(use_default_palette) {
-		palette = read_default_palette();
+		use_palette = true;
+		palette = Palette::createDefault();
 		do_shade = 1;
 		out_numbands = 3;
-	} else if(palette_fn) {
-		palette = read_palette_file(palette_fn);
-		if(!palette) fatal_error("palette was null"); // this can't happen
+	} else if(palette_fn.size()) {
+		use_palette = true;
+		palette = Palette::fromFile(palette_fn);
 		do_shade = 1;
 		out_numbands = 3;
 	} else if(alpha_overlay) {
@@ -229,9 +223,11 @@ int main(int argc, char *argv[]) {
 		out_numbands = 1;
 	}
 
+	if(use_palette) assert(out_numbands == 3);
+
 	//////// open DEM ////////
 
-	GDALDatasetH src_ds = GDALOpen(src_fn, GA_ReadOnly);
+	GDALDatasetH src_ds = GDALOpen(src_fn.c_str(), GA_ReadOnly);
 	if(!src_ds) fatal_error("open failed");
 
 	GDALRasterBandH src_band = GDALGetRasterBand(src_ds, band_id);
@@ -241,16 +237,18 @@ int main(int argc, char *argv[]) {
 	if(!w || !h) fatal_error("missing width/height");
 	printf("Input size is %zd, %zd\n", w, h);
 
-	georef_t georef = init_georef(&geo_opts, src_ds);
+	GeoRef georef = GeoRef(geo_opts, src_ds);
 
 	//////// compute orientation ////////
 
-	double *constant_invaffine = NULL;
+	std::vector<double> constant_invaffine;
+	bool use_constant_invaffine = false;
 	if(do_shade) {
 		if(!georef.fwd_xform) {
-			if(!georef.inv_affine) fatal_error("please specify resolution of image");
+			if(!georef.hasAffine()) fatal_error("please specify resolution of image");
 			printf("warning: no SRS available - basing orientation on affine transform\n");
-			constant_invaffine = MYALLOC(double, 4);
+			use_constant_invaffine = true;
+			constant_invaffine.resize(4);
 			constant_invaffine[0] = georef.inv_affine[1];
 			constant_invaffine[1] = georef.inv_affine[2];
 			constant_invaffine[2] = georef.inv_affine[4];
@@ -261,9 +259,9 @@ int main(int argc, char *argv[]) {
 	//////// open texture ////////
 
 	GDALDatasetH tex_ds = NULL;
-	GDALRasterBandH *tex_bands = NULL;
-	if(tex_fn) {
-		tex_ds = GDALOpen(tex_fn, GA_ReadOnly);
+	std::vector<GDALRasterBandH> tex_bands;
+	if(tex_fn.size()) {
+		tex_ds = GDALOpen(tex_fn.c_str(), GA_ReadOnly);
 		if(!tex_ds) fatal_error("open failed");
 
 		size_t tex_w = GDALGetRasterXSize(tex_ds);
@@ -272,42 +270,44 @@ int main(int argc, char *argv[]) {
 		if(tex_w != w || tex_h != h) fatal_error("DEM and texture are different sizes");
 
 		out_numbands = GDALGetRasterCount(tex_ds);
-		tex_bands = MYALLOC(GDALRasterBandH, out_numbands);
+		if(!out_numbands) fatal_error("texture file had no bands");
 		for(int i=0; i<out_numbands; i++) {
-			tex_bands[i] = GDALGetRasterBand(tex_ds, i+1);
-			if(!tex_bands[i]) fatal_error("could not open texture band");
+			GDALRasterBandH b = GDALGetRasterBand(tex_ds, i+1);
+			if(!b) fatal_error("could not open texture band");
+			tex_bands.push_back(b);
 		}
 	}
 
 	//////// open output ////////
 
-	GDALDriverH dst_driver = GDALGetDriverByName(output_format);
-	if(!dst_driver) fatal_error("unrecognized output format (%s)", output_format);
-	GDALDatasetH dst_ds = GDALCreate(dst_driver, dst_fn, w, h, out_numbands, GDT_Byte, NULL);
+	GDALDriverH dst_driver = GDALGetDriverByName(output_format.c_str());
+	if(!dst_driver) fatal_error("unrecognized output format (%s)", output_format.c_str());
+	GDALDatasetH dst_ds = GDALCreate(
+		dst_driver, dst_fn.c_str(), w, h, out_numbands, GDT_Byte, NULL);
 	if(!dst_ds) fatal_error("couldn't create dst_dataset");
 
-	if(georef.fwd_affine) {
-		GDALSetGeoTransform(dst_ds, georef.fwd_affine);
+	if(georef.hasAffine()) {
+		GDALSetGeoTransform(dst_ds, &georef.fwd_affine[0]);
 	}
 	GDALSetProjection(dst_ds, GDALGetProjectionRef(src_ds));
 
-	GDALRasterBandH *dst_band = MYALLOC(GDALRasterBandH, out_numbands);
+	std::vector<GDALRasterBandH> dst_band;
 	for(int i=0; i<out_numbands; i++) {
-		dst_band[i] = GDALGetRasterBand(dst_ds, i+1);
+		dst_band.push_back(GDALGetRasterBand(dst_ds, i+1));
 	}
 
 	//////// setup shade table ////////
 
-	double **shade_table = NULL;
-	double **spec_table = NULL;
+	std::vector<std::vector<double> > shade_table;
+	std::vector<std::vector<double> > spec_table;
 	double ALPHA_THRESH = .5F;
 	double thresh_brite = 1;
 	if(do_shade) {
-		shade_table = MYALLOC(double *, SHADE_TABLE_SIZE*2+1);
-		spec_table  = MYALLOC(double *, SHADE_TABLE_SIZE*2+1);
+		shade_table.resize(SHADE_TABLE_SIZE*2+1);
+		spec_table .resize(SHADE_TABLE_SIZE*2+1);
 		for(int i=0; i<SHADE_TABLE_SIZE*2+1; i++) {
-			shade_table[i] = MYALLOC(double, SHADE_TABLE_SIZE*2+1);
-			spec_table [i] = MYALLOC(double, SHADE_TABLE_SIZE*2+1);
+			shade_table[i].resize(SHADE_TABLE_SIZE*2+1);
+			spec_table [i].resize(SHADE_TABLE_SIZE*2+1);
 		}
 		double lightvec_len = sqrt(
 			lightvec[0] * lightvec[0] +
@@ -353,27 +353,30 @@ int main(int argc, char *argv[]) {
 
 	//////// process image ////////
 
-	if(!ndv_def.nranges) {
-		add_ndv_from_raster(&ndv_def, src_ds, 1, &band_id);
+	if(ndv_def.empty()) {
+		std::vector<size_t> ndv_bandids;
+		ndv_bandids.push_back(band_id);
+		ndv_def = NdvDef(src_ds, ndv_bandids);
 	}
 
-	double *inbuf_prev = MYALLOC(double, w);
-	double *inbuf_this = MYALLOC(double, w);
-	double *inbuf_next = MYALLOC(double, w);
-	uint8_t *inbuf_ndv_prev = MYALLOC(uint8_t, w);
-	uint8_t *inbuf_ndv_this = MYALLOC(uint8_t, w);
-	uint8_t *inbuf_ndv_next = MYALLOC(uint8_t, w);
+	std::vector<double> inbuf_prev(w);
+	std::vector<double> inbuf_this(w);
+	std::vector<double> inbuf_next(w);
+	std::vector<uint8_t> inbuf_ndv_prev(w);
+	std::vector<uint8_t> inbuf_ndv_this(w);
+	std::vector<uint8_t> inbuf_ndv_next(w);
 
-	uint8_t **outbuf = MYALLOC(uint8_t *, out_numbands);
+	std::vector<std::vector<uint8_t> > outbuf(out_numbands);
 	for(int i=0; i<out_numbands; i++) {
-		outbuf[i] = MYALLOC(uint8_t, w);
+		outbuf[i].resize(w);
 	}
-	uint8_t *pixel = MYALLOC(uint8_t, out_numbands);
+	std::vector<uint8_t> pixel(out_numbands);
 
-	double *invaffine_tierow_above=NULL, *invaffine_tierow_below=NULL;
-	if(do_shade && !constant_invaffine) {
-		invaffine_tierow_above = MYALLOC(double, w * 4);
-		invaffine_tierow_below = MYALLOC(double, w * 4);
+	std::vector<double> invaffine_tierow_above;
+	std::vector<double> invaffine_tierow_below;
+	if(do_shade && !use_constant_invaffine) {
+		invaffine_tierow_above.resize(w * 4);
+		invaffine_tierow_below.resize(w * 4);
 	}
 
 	double min=0, max=0; // initialized to prevent compiler warning
@@ -381,53 +384,52 @@ int main(int argc, char *argv[]) {
 	for(size_t row=0; row<h; row++) {
 		GDALTermProgress((double)row / (double)h, NULL, NULL);
 		if(row == 0) {
-			GDALRasterIO(src_band, GF_Read, 0, 0, w, 1, inbuf_prev, w, 1, GDT_Float64, 0, 0);
-			GDALRasterIO(src_band, GF_Read, 0, 0, w, 1, inbuf_this, w, 1, GDT_Float64, 0, 0);
-			GDALRasterIO(src_band, GF_Read, 0, 1, w, 1, inbuf_next, w, 1, GDT_Float64, 0, 0);
-			array_check_ndv(&ndv_def, 0, inbuf_prev, NULL, inbuf_ndv_prev, w);
-			array_check_ndv(&ndv_def, 0, inbuf_this, NULL, inbuf_ndv_this, w);
-			array_check_ndv(&ndv_def, 0, inbuf_next, NULL, inbuf_ndv_next, w);
-			scale_values(inbuf_prev, w, src_scale, src_offset);
-			scale_values(inbuf_this, w, src_scale, src_offset);
-			scale_values(inbuf_next, w, src_scale, src_offset);
+			GDALRasterIO(src_band, GF_Read, 0, 0, w, 1, &inbuf_prev[0], w, 1, GDT_Float64, 0, 0);
+			GDALRasterIO(src_band, GF_Read, 0, 0, w, 1, &inbuf_this[0], w, 1, GDT_Float64, 0, 0);
+			GDALRasterIO(src_band, GF_Read, 0, 1, w, 1, &inbuf_next[0], w, 1, GDT_Float64, 0, 0);
+			ndv_def.arrayCheckNdv(0, &inbuf_prev[0], &inbuf_ndv_prev[0], w);
+			ndv_def.arrayCheckNdv(0, &inbuf_this[0], &inbuf_ndv_this[0], w);
+			ndv_def.arrayCheckNdv(0, &inbuf_next[0], &inbuf_ndv_next[0], w);
+			scale_values(&inbuf_prev[0], w, src_scale, src_offset);
+			scale_values(&inbuf_this[0], w, src_scale, src_offset);
+			scale_values(&inbuf_next[0], w, src_scale, src_offset);
 		} else {
-			double *swapd = inbuf_prev;
-			inbuf_prev = inbuf_this;
-			inbuf_this = inbuf_next;
-			inbuf_next = swapd;
-			uint8_t *swapc = inbuf_ndv_prev;
-			inbuf_ndv_prev = inbuf_ndv_this;
-			inbuf_ndv_this = inbuf_ndv_next;
-			inbuf_ndv_next = swapc;
-			if(row == h-1) {
-				GDALRasterIO(src_band, GF_Read, 0, row, w, 1, inbuf_next, w, 1, GDT_Float64, 0, 0);
-			} else {
-				GDALRasterIO(src_band, GF_Read, 0, row+1, w, 1, inbuf_next, w, 1, GDT_Float64, 0, 0);
-			}
-			array_check_ndv(&ndv_def, 0, inbuf_next, NULL, inbuf_ndv_next, w);
-			scale_values(inbuf_next, w, src_scale, src_offset);
+			// rotate buffers
+			std::vector<double> swapd;
+			std::swap(swapd, inbuf_prev);
+			std::swap(inbuf_prev, inbuf_this);
+			std::swap(inbuf_this, inbuf_next);
+			std::swap(inbuf_next, swapd);
+			std::vector<uint8_t> swapc;
+			std::swap(swapc, inbuf_ndv_prev);
+			std::swap(inbuf_ndv_prev, inbuf_ndv_this);
+			std::swap(inbuf_ndv_this, inbuf_ndv_next);
+			std::swap(inbuf_ndv_next, swapc);
+
+			int read_row = (row == h-1) ? row : row+1;
+			GDALRasterIO(src_band, GF_Read, 0, read_row, w, 1, &inbuf_next[0], w, 1, GDT_Float64, 0, 0);
+			ndv_def.arrayCheckNdv(0, &inbuf_next[0], &inbuf_ndv_next[0], w);
+			scale_values(&inbuf_next[0], w, src_scale, src_offset);
 		}
-		if(tex_bands) {
+		if(tex_bands.size()) {
 			for(int i=0; i<out_numbands; i++) {
-				GDALRasterIO(tex_bands[i], GF_Read, 0, row, w, 1, outbuf[i], w, 1, GDT_Byte, 0, 0);
+				GDALRasterIO(tex_bands[i], GF_Read, 0, row, w, 1, &outbuf[i][0], w, 1, GDT_Byte, 0, 0);
 			}
 		}
 
 		double grid_fraction = 0;
-		if(do_shade && !constant_invaffine) {
+		if(do_shade && !use_constant_invaffine) {
 			// the part sets up the bilinear interpolation of invaffine
 			int above_tiept = grid_spacing * (row / grid_spacing);
 			int below_tiept = above_tiept + grid_spacing;
 			if(below_tiept > (int)h) below_tiept = (int)h;
 			if(row == (size_t)above_tiept) {
 				if(row == 0) {
-					compute_tierow_invaffine(&georef, w, 0, grid_spacing, invaffine_tierow_above);
+					compute_tierow_invaffine(georef, w, 0, grid_spacing, &invaffine_tierow_above[0]);
 				} else {
-					double *tmp = invaffine_tierow_above;
-					invaffine_tierow_above = invaffine_tierow_below;
-					invaffine_tierow_below = tmp;
+					std::swap(invaffine_tierow_above, invaffine_tierow_below);
 				}
-				compute_tierow_invaffine(&georef, w, below_tiept, grid_spacing, invaffine_tierow_below);
+				compute_tierow_invaffine(georef, w, below_tiept, grid_spacing, &invaffine_tierow_below[0]);
 			}
 			double segment_height = below_tiept - above_tiept;
 			grid_fraction = ((double)row - (double)above_tiept) / segment_height;
@@ -468,7 +470,7 @@ int main(int argc, char *argv[]) {
 				//double dy2 = invaffine_c * dx + invaffine_d * (-dy);
 				
 				double invaffine[4];
-				if(constant_invaffine) {
+				if(use_constant_invaffine) {
 					for(int i=0; i<4; i++) invaffine[i] = constant_invaffine[i];
 				} else {
 					for(int i=0; i<4; i++) {
@@ -497,9 +499,10 @@ int main(int argc, char *argv[]) {
 				spec = 0.0;
 			}
 			if(inbuf_ndv_this[col]) {
-				if(palette) {
-					get_nan_color(pixel, palette);
-					for(int i=0; i<out_numbands; i++) outbuf[i][col] = pixel[i];
+				if(use_palette) {
+					outbuf[0][col] = palette.nan_color.r;
+					outbuf[1][col] = palette.nan_color.g;
+					outbuf[2][col] = palette.nan_color.b;
 				} else {
 					for(int i=0; i<out_numbands; i++) outbuf[i][col] = 0;
 				}
@@ -536,8 +539,11 @@ int main(int argc, char *argv[]) {
 					pixel[0] = pixel[1] = pixel[2] = (uint8_t)(255.0 * white);
 					pixel[3] = (uint8_t)(255.0 * alpha);
 				} else {
-					if(palette) {
-						get_palette_color(pixel, val, palette);
+					if(use_palette) {
+						RGB c = palette.get(val);
+						pixel[0] = c.r;
+						pixel[1] = c.g;
+						pixel[2] = c.b;
 					} else if(tex_ds) {
 						for(int i=0; i<out_numbands; i++) pixel[i] = outbuf[i][col];
 					} else {
@@ -559,7 +565,7 @@ int main(int argc, char *argv[]) {
 			}
 		}
 		for(int i=0; i<out_numbands; i++) {
-			GDALRasterIO(dst_band[i], GF_Write, 0, row, w, 1, outbuf[i], w, 1, GDT_Byte, 0, 0);
+			GDALRasterIO(dst_band[i], GF_Write, 0, row, w, 1, &outbuf[i][0], w, 1, GDT_Byte, 0, 0);
 		}
 	}
 
@@ -586,105 +592,11 @@ void scale_values(double *vals, size_t w, double scale, double offset) {
 	}
 }
 
-palette_t *parse_palette(const char * const *lines) {
-	palette_t *p = MYALLOC(palette_t, 1);
-
-	int num = 0;
-	p->vals = NULL;
-	p->reds = p->greens = p->blues = NULL;
-	p->nan_red = p->nan_green = p->nan_blue = 0;
-
-	for(int line_num=0; lines[line_num]; line_num++) {
-		const char *line = lines[line_num];
-		uint8_t r, g, b;
-		double val;
-		if(4 != sscanf(line, "%lf %hhd %hhd %hhd\n", &val, &r, &g, &b)) {
-			fatal_error("cannot parse line in palette file: [%s]", line);
-		}
-		if(isnan(val)) {
-			p->nan_red   = r;
-			p->nan_green = g;
-			p->nan_blue  = b;
-		} else {
-			num++;
-			p->vals   = REMYALLOC(double, p->vals, num);
-			p->reds   = REMYALLOC(uint8_t, p->reds, num);
-			p->greens = REMYALLOC(uint8_t, p->greens, num);
-			p->blues  = REMYALLOC(uint8_t, p->blues, num);
-			p->vals  [num-1] = val;
-			p->reds  [num-1] = r;
-			p->greens[num-1] = g;
-			p->blues [num-1] = b;
-		}
-	}
-	
-	if(num < 2) fatal_error("not enough entries in palette");
-	p->num_vals = num;
-
-	return p;
-}
-
-palette_t *read_palette_file(const char *fn) {
-	FILE *fh = fopen(fn, "r");
-	if(!fh) fatal_error("cannot open palette file");
-	char **lines = NULL;
-	int num_lines = 0;
-	char *line;
-	char buf[1000];
-	while((line = fgets(buf, 1000, fh))) {
-		lines = REMYALLOC(char *, lines, (num_lines + 2));
-		lines[num_lines] = strdup(line);
-		lines[num_lines+1] = NULL;
-		num_lines++;
-	}
-	fclose(fh);
-
-	if(!num_lines) fatal_error("palette file was empty");
-
-	palette_t *p = parse_palette(lines);
-
-	for(int i=0; lines[i]; i++) free(lines[i]);
-	free(lines);
-
-	return p;
-}
-
-palette_t *read_default_palette() {
-	return parse_palette(DEFAULT_PALETTE);
-}
-
-void get_nan_color(uint8_t *buf, palette_t *pal) {
-	buf[0] = pal->nan_red;
-	buf[1] = pal->nan_green;
-	buf[2] = pal->nan_blue;
-}
-
-void get_palette_color(uint8_t *buf, double val, palette_t *pal) {
-	double v1, v2, alpha;
-
-	if(val < pal->vals[0]) val = pal->vals[0];
-	if(val > pal->vals[pal->num_vals-1]) val = pal->vals[pal->num_vals-1];
-
-	for(int i=0; i<pal->num_vals-1; i++) {
-		v1 = pal->vals[i];
-		v2 = pal->vals[i+1];
-		if(val >= v1 && val <= v2) {
-			alpha = (val - v1) / (v2 - v1);
-			buf[0] = (uint8_t)((double)pal->reds  [i]*(1.0-alpha) + (double)pal->reds  [i+1]*alpha + .5);
-			buf[1] = (uint8_t)((double)pal->greens[i]*(1.0-alpha) + (double)pal->greens[i+1]*alpha + .5);
-			buf[2] = (uint8_t)((double)pal->blues [i]*(1.0-alpha) + (double)pal->blues [i+1]*alpha + .5);
-			return;
-		}
-	}
-
-	fatal_error("palette file out of sequence, can't find val %g\n", val);
-}
-
 // this function generates a 2x2 matrix that
 // can be used to convert row/column gradients
 // to easting/northing gradients
 void compute_invaffine(
-	georef_t *georef, double col, double row, double *invaffine
+	const GeoRef &georef, double col, double row, double *invaffine
 ) {
 	// in case we return due to error:
 	invaffine[0] = invaffine[1] =
@@ -697,9 +609,9 @@ void compute_invaffine(
 	double lon_dx, lat_dx;
 	double lon_dy, lat_dy;
 	bool error =
-		xy2ll(georef, col, row, &lon_0, &lat_0) ||
-		xy2ll(georef, col+epsilon, row, &lon_dx, &lat_dx) ||
-		xy2ll(georef, col, row+epsilon, &lon_dy, &lat_dy);
+		georef.xy2ll(col, row, &lon_0, &lat_0) ||
+		georef.xy2ll(col+epsilon, row, &lon_dx, &lat_dx) ||
+		georef.xy2ll(col, row+epsilon, &lon_dy, &lat_dy);
 
 	if(error) {
 		invaffine[0] = 0;
@@ -728,10 +640,11 @@ void compute_invaffine(
 	lon_dy /= epsilon; lat_dy /= epsilon;
 
 	// convert lon/lat infinitesimals to true east/north infinitesimals
-	double te_dx = cos(lat_0 * D2R) * lon_dx * D2R * EARTH_RADIUS;
-	double tn_dx = lat_dx * D2R * EARTH_RADIUS;
-	double te_dy = cos(lat_0 * D2R) * lon_dy * D2R * EARTH_RADIUS;
-	double tn_dy = lat_dy * D2R * EARTH_RADIUS;
+	double planet_radius = georef.have_semi_major ? georef.semi_major : EARTH_RADIUS;
+	double te_dx = cos(lat_0 * D2R) * lon_dx * D2R * planet_radius;
+	double tn_dx = lat_dx * D2R * planet_radius;
+	double te_dy = cos(lat_0 * D2R) * lon_dy * D2R * planet_radius;
+	double tn_dy = lat_dy * D2R * planet_radius;
 
 	// invert the affine matrix:
 
@@ -759,7 +672,7 @@ void compute_invaffine(
 
 // interpolate the invaffine for an entire row
 void compute_tierow_invaffine(
-	georef_t *georef,
+	const GeoRef &georef,
 	int num_cols, int row, int grid_spacing,
 	double *invaffine_tierow
 ) {
