@@ -26,6 +26,8 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <boost/lexical_cast.hpp>
+
 #include "common.h"
 #include "polygon.h"
 #include "polygon-rasterizer.h"
@@ -35,13 +37,15 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 #include "mask.h"
 #include "rectangle_finder.h"
 
-void usage(const char *cmdname) {
-	printf("Usage:\n  %s [options] [image_name]\n", cmdname);
+using namespace dangdal;
+
+void usage(const std::string &cmdname) {
+	printf("Usage:\n  %s [options] [image_name]\n", cmdname.c_str());
 	printf("\n");
 	
-	print_georef_usage();
+	GeoOpts::printUsage();
 	printf("\n");
-	print_ndv_usage();
+	NdvDef::printUsage();
 
 	printf("\
 \n\
@@ -69,18 +73,19 @@ Examples:\n\
 }
 
 int main(int argc, char **argv) {
-	char *input_raster_fn = NULL;
+	const std::string cmdname = argv[0];
+	if(argc == 1) usage(cmdname);
+	std::vector<std::string> arg_list = argv_to_list(argc, argv);
+
+	std::string input_raster_fn;
 
 	bool inspect_rect4 = 0;
 	bool fuzzy_match = 0;
-	char *debug_report = NULL;
-	char *mask_out_fn = NULL;
-	int inspect_numbands = 0;
-	int *inspect_bandids = NULL;
+	std::string debug_report;
+	std::string mask_out_fn;
+	std::vector<size_t> inspect_bandids;
 	bool do_erosion = 0;
 
-	if(argc == 1) usage(argv[0]);
-
 	// We will be sending YAML to stdout, so stuff that would normally
 	// go to stdout (such as debug messages or progress bars) should
 	// go to stderr.
@@ -89,86 +94,88 @@ int main(int argc, char **argv) {
 	close(1);
 	dup2(2, 1);
 
-	geo_opts_t geo_opts = init_geo_options(&argc, &argv);
-	ndv_def_t ndv_def = init_ndv_options(&argc, &argv);
+	GeoOpts geo_opts = GeoOpts(arg_list);
+	NdvDef ndv_def = NdvDef(arg_list);
 
-	int argp = 1;
-	while(argp < argc) {
-		char *arg = argv[argp++];
+	size_t argp = 1;
+	while(argp < arg_list.size()) {
+		const std::string &arg = arg_list[argp++];
 		// FIXME - check duplicate values
 		if(arg[0] == '-') {
-			if(!strcmp(arg, "-v")) {
-				VERBOSE++;
-			} else if(!strcmp(arg, "-inspect-rect4")) {
-				inspect_rect4 = 1;
-			} else if(!strcmp(arg, "-fuzzy-match")) {
-				fuzzy_match = 1;
-			} else if(!strcmp(arg, "-b")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				int bandid = strtol(argv[argp++], &endptr, 10);
-				if(*endptr) usage(argv[0]);
-				inspect_bandids = REMYALLOC(int, inspect_bandids, (inspect_numbands+1));
-				inspect_bandids[inspect_numbands++] = bandid;
-			} else if(!strcmp(arg, "-erosion")) {
-				do_erosion = 1;
-			} else if(!strcmp(arg, "-report")) {
-				if(argp == argc) usage(argv[0]);
-				debug_report = argv[argp++];
-			} else if(!strcmp(arg, "-mask-out")) {
-				if(argp == argc) usage(argv[0]);
-				mask_out_fn = argv[argp++];
-			} else usage(argv[0]);
+			try {
+				if(arg == "-v") {
+					VERBOSE++;
+				} else if(arg == "-inspect-rect4") {
+					inspect_rect4 = 1;
+				} else if(arg == "-fuzzy-match") {
+					fuzzy_match = 1;
+				} else if(arg == "-b") {
+					if(argp == arg_list.size()) usage(cmdname);
+					int bandid = boost::lexical_cast<int>(arg_list[argp++]);
+					inspect_bandids.push_back(bandid);
+				} else if(arg == "-erosion") {
+					do_erosion = 1;
+				} else if(arg == "-report") {
+					if(argp == arg_list.size()) usage(cmdname);
+					debug_report = arg_list[argp++];
+				} else if(arg == "-mask-out") {
+					if(argp == arg_list.size()) usage(cmdname);
+					mask_out_fn = arg_list[argp++];
+				} else {
+					usage(cmdname);
+				}
+			} catch(boost::bad_lexical_cast &e) {
+				fatal_error("cannot parse number given on command line");
+			}
 		} else {
-			if(input_raster_fn) usage(argv[0]);
+			if(input_raster_fn.size()) usage(cmdname);
 			input_raster_fn = arg;
 		}
 	}
 
 	bool do_inspect = inspect_rect4;
-	if(do_inspect && !input_raster_fn) fatal_error("must specify filename of image");
+	if(do_inspect && input_raster_fn.empty()) fatal_error("must specify filename of image");
 
 	GDALAllRegister();
 
 	GDALDatasetH ds = NULL;
-	if(input_raster_fn) {
-		ds = GDALOpen(input_raster_fn, GA_ReadOnly);
+	if(input_raster_fn.size()) {
+		ds = GDALOpen(input_raster_fn.c_str(), GA_ReadOnly);
 		if(!ds) fatal_error("open failed");
 	}
 
-	if(do_inspect && !inspect_numbands) {
-		inspect_numbands = GDALGetRasterCount(ds);
-		inspect_bandids = MYALLOC(int, inspect_numbands);
-		for(int i=0; i<inspect_numbands; i++) inspect_bandids[i] = i+1;
+	if(do_inspect && inspect_bandids.empty()) {
+		size_t nbands = GDALGetRasterCount(ds);
+		for(size_t i=0; i<nbands; i++) inspect_bandids.push_back(i+1);
 	}
 
 	if(!do_inspect) {
-		if(fuzzy_match)      fatal_error("-fuzzy-match option can only be used with -inspect-rect4 option");
-		if(ndv_def.nranges)  fatal_error("NDV options can only be used with -inspect-rect4 option");
-		if(debug_report)     fatal_error("-report option can only be used with -inspect-rect4 option");
-		if(mask_out_fn)      fatal_error("-mask-out option can only be used with -inspect-rect4 option");
-		if(inspect_numbands) fatal_error("-b option can only be used with -inspect-rect4 option");
-		if(do_erosion)       fatal_error("-erosion option can only be used with -inspect-rect4 option");
+		std::string suffix(" can only be used with -inspect-rect4 option");
+		if(fuzzy_match)            fatal_error("-fuzzy-match option"+suffix);
+		if(!ndv_def.empty())       fatal_error("NDV options"+suffix);
+		if(debug_report.size())    fatal_error("-report option"+suffix);
+		if(mask_out_fn.size())     fatal_error("-mask-out option"+suffix);
+		if(inspect_bandids.size()) fatal_error("-b option"+suffix);
+		if(do_erosion)             fatal_error("-erosion option"+suffix);
 	}
 
 	CPLPushErrorHandler(CPLQuietErrorHandler);
 
-	georef_t georef = init_georef(&geo_opts, ds);
+	GeoRef georef = GeoRef(geo_opts, ds);
 
-	report_image_t *dbuf = NULL;
+	DebugPlot *dbuf = NULL;
 	BitGrid mask(0, 0);
 	if(do_inspect) {
-		if(!ndv_def.nranges) {
-			add_ndv_from_raster(&ndv_def, ds, inspect_numbands, inspect_bandids);
+		if(ndv_def.empty()) {
+			ndv_def = NdvDef(ds, inspect_bandids);
 		}
 
-		if(debug_report) {
-			dbuf = create_plot(georef.w, georef.h);
+		if(debug_report.size()) {
+			dbuf = new DebugPlot(georef.w, georef.h);
 			dbuf->mode = PLOT_RECT4;
 		}
 
-		mask = get_bitgrid_for_dataset(ds, inspect_numbands, inspect_bandids,
-			&ndv_def, dbuf);
+		mask = get_bitgrid_for_dataset(ds, inspect_bandids, ndv_def, dbuf);
 
 		if(do_erosion) {
 			mask.erode();
@@ -183,22 +190,16 @@ int main(int argc, char **argv) {
 
 	if(ds) {
 		int band_count = GDALGetRasterCount(ds);
-		const char *datatypes = "";
+		std::string datatypes;
 		for(int i=0; i<band_count; i++) {
 			GDALRasterBandH band = GDALGetRasterBand(ds, i+1);
 			GDALDataType gdt = GDALGetRasterDataType(band);
 			const char *dt = GDALGetDataTypeName(gdt);
-			if(i) {
-				char *join_str = MYALLOC(char,
-					strlen(datatypes) + 1 + strlen(dt) + 1);
-				sprintf(join_str, "%s,%s", datatypes, dt);
-				datatypes = join_str;
-			} else {
-				datatypes = dt;
-			}
+			if(i) datatypes.append(",");
+			datatypes.append(dt);
 		}
 		fprintf(yaml_fh, "num_bands: %d\n", band_count);
-		fprintf(yaml_fh, "datatype: %s\n", datatypes);
+		fprintf(yaml_fh, "datatype: %s\n", datatypes.c_str());
 
 		char **metadata = GDALGetMetadata(ds, "");
 		if(metadata) {
@@ -209,11 +210,11 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	if(georef.s_srs && strlen(georef.s_srs)) {
-		fprintf(yaml_fh, "s_srs: '%s'\n", georef.s_srs);
+	if(georef.s_srs.size()) {
+		fprintf(yaml_fh, "s_srs: '%s'\n", georef.s_srs.c_str());
 	}
-	if(georef.units_name) {
-		fprintf(yaml_fh, "units_name: '%s'\n", georef.units_name);
+	if(georef.units_name.size()) {
+		fprintf(yaml_fh, "units_name: '%s'\n", georef.units_name.c_str());
 	}
 	if(georef.units_val) {
 		fprintf(yaml_fh, "units_val: %lf\n", georef.units_val);
@@ -224,7 +225,7 @@ int main(int argc, char **argv) {
 	if(georef.res_meters_x && georef.res_meters_y) {
 		fprintf(yaml_fh, "res_meters: %.15f %.15f\n", georef.res_meters_x, georef.res_meters_y);
 	}
-	if(georef.fwd_affine) {
+	if(georef.hasAffine()) {
 		fprintf(yaml_fh, "affine:\n");
 		for(int i=0; i<6; i++) fprintf(yaml_fh, "  - %.15f\n", georef.fwd_affine[i]);
 	}
@@ -232,16 +233,16 @@ int main(int argc, char **argv) {
 	double lon, lat;
 	double east, north;
 
-	vertex_t center;
+	Vertex center;
 	fprintf(yaml_fh, "center:\n");
-	center = (vertex_t){ (double)georef.w/2.0, (double)georef.h/2.0 };
-	if(georef.fwd_xform && georef.fwd_affine) {
-		xy2ll_or_die(&georef, center.x, center.y, &lon, &lat);
+	center = Vertex((double)georef.w/2.0, (double)georef.h/2.0);
+	if(georef.fwd_xform && georef.hasAffine()) {
+		georef.xy2ll_or_die(center.x, center.y, &lon, &lat);
 		fprintf(yaml_fh, "  lon: %.15f\n", lon);
 		fprintf(yaml_fh, "  lat: %.15f\n", lat);
 	}
-	if(georef.fwd_affine) {
-		xy2en(&georef, center.x, center.y, &east, &north);
+	if(georef.hasAffine()) {
+		georef.xy2en(center.x, center.y, &east, &north);
 		fprintf(yaml_fh, "  east: %.15f\n", east);
 		fprintf(yaml_fh, "  north: %.15f\n", north);
 	}
@@ -249,16 +250,16 @@ int main(int argc, char **argv) {
 	fprintf(yaml_fh, "  y: %.15f\n", center.y);
 
 	if(do_inspect) {
-		vertex_t centroid;
+		Vertex centroid;
 		fprintf(yaml_fh, "centroid:\n");
 		centroid = mask.centroid();
-		if(georef.fwd_xform && georef.fwd_affine) {
-			xy2ll_or_die(&georef, centroid.x, centroid.y, &lon, &lat);
+		if(georef.fwd_xform && georef.hasAffine()) {
+			georef.xy2ll_or_die(centroid.x, centroid.y, &lon, &lat);
 			fprintf(yaml_fh, "  lon: %.15f\n", lon);
 			fprintf(yaml_fh, "  lat: %.15f\n", lat);
 		}
-		if(georef.fwd_affine) {
-			xy2en(&georef, centroid.x, centroid.y, &east, &north);
+		if(georef.hasAffine()) {
+			georef.xy2en(centroid.x, centroid.y, &east, &north);
 			fprintf(yaml_fh, "  east: %.15f\n", east);
 			fprintf(yaml_fh, "  north: %.15f\n", north);
 		}
@@ -267,34 +268,32 @@ int main(int argc, char **argv) {
 	}
 
 	if(inspect_rect4) {
-		ring_t rect4 = calc_rect4_from_mask(mask, georef.w, georef.h, dbuf, fuzzy_match);
+		Ring rect4 = calc_rect4_from_mask(mask, georef.w, georef.h, dbuf, fuzzy_match);
 
-		if(rect4.npts != 4) {
+		if(rect4.pts.size() != 4) {
 			fatal_error("could not find four-sided region");
 		}
 
-		if(mask_out_fn) {
-			mpoly_t *bpoly = MYALLOC(mpoly_t, 1);
-			bpoly->num_rings = 1;
-			bpoly->rings = MYALLOC(ring_t, 1);
-			bpoly->rings[0] = rect4;
+		if(mask_out_fn.size()) {
+			Mpoly bpoly;
+			bpoly.rings.push_back(rect4);
 
 			mask_from_mpoly(bpoly, georef.w, georef.h, mask_out_fn);
 		}
 
 		const char *labels[] = { "upper_left", "upper_right", "lower_right", "lower_left" };
-		if(georef.fwd_xform && georef.fwd_affine) {
+		if(georef.fwd_xform && georef.hasAffine()) {
 			fprintf(yaml_fh, "geometry_ll:\n  type: rectangle4\n");
 			for(int i=0; i<4; i++) {
-				xy2ll_or_die(&georef, rect4.pts[i].x, rect4.pts[i].y, &lon, &lat);
+				georef.xy2ll_or_die(rect4.pts[i].x, rect4.pts[i].y, &lon, &lat);
 				fprintf(yaml_fh, "  %s_lon: %.15f\n", labels[i], lon);
 				fprintf(yaml_fh, "  %s_lat: %.15f\n", labels[i], lat);
 			}
 		}
-		if(georef.fwd_affine) {
+		if(georef.hasAffine()) {
 			fprintf(yaml_fh, "geometry_en:\n  type: rectangle4\n");
 			for(int i=0; i<4; i++) {
-				xy2en(&georef, rect4.pts[i].x, rect4.pts[i].y, &east, &north);
+				georef.xy2en(rect4.pts[i].x, rect4.pts[i].y, &east, &north);
 				fprintf(yaml_fh, "  %s_east: %.15f\n", labels[i], east);
 				fprintf(yaml_fh, "  %s_north: %.15f\n", labels[i], north);
 			}
@@ -309,20 +308,20 @@ int main(int argc, char **argv) {
 		double e_pos[] = { 0, (double)georef.w/2.0, georef.w };
 		const char *n_labels[] = { "upper", "mid", "lower" };
 		double n_pos[] = { 0, (double)georef.h/2.0, georef.h };
-		if(georef.fwd_xform && georef.fwd_affine) {
+		if(georef.fwd_xform && georef.hasAffine()) {
 			fprintf(yaml_fh, "geometry_ll:\n  type: rectangle8\n");
 			for(int i=0; i<3; i++) for(int j=0; j<3; j++) {
 				if(!strcmp(e_labels[i], "mid") && !strcmp(n_labels[j], "mid")) continue;
-				xy2ll_or_die(&georef, e_pos[i], n_pos[j], &lon, &lat);
+				georef.xy2ll_or_die(e_pos[i], n_pos[j], &lon, &lat);
 				fprintf(yaml_fh, "  %s_%s_lon: %.15f\n", n_labels[j], e_labels[i], lon);
 				fprintf(yaml_fh, "  %s_%s_lat: %.15f\n", n_labels[j], e_labels[i], lat);
 			}
 		}
-		if(georef.fwd_affine) {
+		if(georef.hasAffine()) {
 			fprintf(yaml_fh, "geometry_en:\n  type: rectangle8\n");
 			for(int i=0; i<3; i++) for(int j=0; j<3; j++) {
 				if(!strcmp(e_labels[i], "mid") && !strcmp(n_labels[j], "mid")) continue;
-				xy2en(&georef, e_pos[i], n_pos[j], &east, &north);
+				georef.xy2en(e_pos[i], n_pos[j], &east, &north);
 				fprintf(yaml_fh, "  %s_%s_east: %.15f\n", n_labels[j], e_labels[i], east);
 				fprintf(yaml_fh, "  %s_%s_north: %.15f\n", n_labels[j], e_labels[i], north);
 			}
@@ -335,7 +334,7 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	if(dbuf) write_plot(dbuf, debug_report);
+	if(dbuf) dbuf->writePlot(debug_report);
 
 	CPLPopErrorHandler();
 
