@@ -26,54 +26,60 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <vector>
+
 #include "common.h"
 
-void usage(const char *cmdname) {
+void usage(const std::string &cmdname) {
 	printf("Usage:\n");
-	printf("    %s -in <rgb.tif> -in <mask.tif> -out <out.vrt>\n", cmdname);
+	printf("    %s -in <rgb.tif> -in <mask.tif> -out <out.vrt>\n", cmdname.c_str());
 	printf("\nMerges several images into one image with many bands.\n");
+	printf("This program is obsoleted by \"gdalbuildvrt -separate\" from GDAL 1.7.\n");
 	exit(1);
 }
 
 int main(int argc, char *argv[]) {
-	char *dst_fn = NULL;
+	const std::string cmdname = argv[0];
+	if(argc == 1) usage(cmdname);
+	std::vector<std::string> arg_list = argv_to_list(argc, argv);
 
-	int src_ds_count = 0;
-	char **src_fn = NULL;
+	std::string dst_fn;
+	std::vector<std::string> src_fn;
 
 	GDALAllRegister();
 
-	int argp = 1;
-	while(argp < argc) {
-		char *arg = argv[argp++];
+	size_t argp = 1;
+	while(argp < arg_list.size()) {
+		const std::string &arg = arg_list[argp++];
 		// FIXME - check duplicate values
 		if(arg[0] == '-') {
-			if(!strcmp(arg, "-out")) { if(argp == argc) usage(argv[0]); dst_fn = argv[argp++]; }
-			else if(!strcmp(arg, "-in")) {
-				if(argp == argc) usage(argv[0]);
-				char *fn = argv[argp++];
-				src_fn = REMYALLOC(char *, src_fn, (src_ds_count+1));
-				src_fn[src_ds_count] = fn; 
-				src_ds_count++;
+			if(arg == "-out") {
+				if(argp == arg_list.size()) usage(cmdname);
+				dst_fn = arg_list[argp++];
+			} else if(arg == "-in") {
+				if(argp == arg_list.size()) usage(cmdname);
+				src_fn.push_back(arg_list[argp++]);
+			} else {
+				usage(cmdname);
 			}
-			else usage(argv[0]);
 		} else {
-			usage(argv[0]);
+			usage(cmdname);
 		}
 	}
 
-	if(src_ds_count < 1) usage(argv[0]);
-	if(!dst_fn) usage(argv[0]);
+	if(src_fn.empty()) usage(cmdname);
+	if(dst_fn.empty()) usage(cmdname);
 
-	GDALDatasetH *src_ds = MYALLOC(GDALDatasetH, src_ds_count);
+	std::vector<GDALDatasetH> src_ds;
 
 	size_t w=0, h=0;
-	for(int ds_idx=0; ds_idx<src_ds_count; ds_idx++) {
-		src_ds[ds_idx] = GDALOpen(src_fn[ds_idx], GA_ReadOnly);
-		if(!src_ds[ds_idx]) fatal_error("open failed");
+	for(size_t ds_idx=0; ds_idx<src_fn.size(); ds_idx++) {
+		GDALDatasetH ds = GDALOpen(src_fn[ds_idx].c_str(), GA_ReadOnly);
+		if(!ds) fatal_error("open failed");
+		src_ds.push_back(ds);
 
-		size_t ds_w = GDALGetRasterXSize(src_ds[ds_idx]);
-		size_t ds_h = GDALGetRasterYSize(src_ds[ds_idx]);
+		size_t ds_w = GDALGetRasterXSize(ds);
+		size_t ds_h = GDALGetRasterYSize(ds);
 		if(!ds_w || !ds_h) fatal_error("missing width/height");
 
 		if(ds_idx) {
@@ -86,11 +92,11 @@ int main(int argc, char *argv[]) {
 
 	GDALDriverH dst_driver = GDALGetDriverByName("VRT");
 	if(!dst_driver) fatal_error("unrecognized output format (VRT)");
-	GDALDatasetH dst_ds = GDALCreateCopy(dst_driver, dst_fn, src_ds[0], 0, NULL, NULL, NULL);
+	GDALDatasetH dst_ds = GDALCreateCopy(dst_driver, dst_fn.c_str(), src_ds[0], 0, NULL, NULL, NULL);
 	if(!dst_ds) fatal_error("could not create output");
 
 	int band_idx = GDALGetRasterCount(src_ds[0]);
-	for(int ds_idx=1; ds_idx<src_ds_count; ds_idx++) {
+	for(size_t ds_idx=1; ds_idx<src_fn.size(); ds_idx++) {
 		int nb = GDALGetRasterCount(src_ds[ds_idx]);
 
 		GDALDatasetH src_vrt_ds = GDALCreateCopy(dst_driver, "", src_ds[ds_idx], 0, NULL, NULL, NULL);
@@ -114,7 +120,7 @@ int main(int argc, char *argv[]) {
 		GDALClose(src_vrt_ds);
 	}
 
-	for(int ds_idx=0; ds_idx<src_ds_count; ds_idx++) {
+	for(size_t ds_idx=0; ds_idx<src_fn.size(); ds_idx++) {
 		GDALClose(src_ds[ds_idx]);
 	}
 	GDALClose(dst_ds);
