@@ -26,10 +26,14 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <vector>
+
+#include <boost/lexical_cast.hpp>
+
 #include "common.h"
 
-void usage(const char *cmdname) {
-	printf("Usage: %s\n", cmdname);
+void usage(const std::string &cmdname) {
+	printf("Usage: %s\n", cmdname.c_str());
 	printf("\t-wh <width> <height>\n");
 	printf("\t-origin <left easting> <top northing>\n");
 	printf("\t-res <pixel_size>\n");
@@ -41,82 +45,78 @@ void usage(const char *cmdname) {
 }
 
 int main(int argc, char *argv[]) {
-	char *src_fn = NULL;
-	char *dst_fn = NULL;
+	const std::string cmdname = argv[0];
+	if(argc == 1) usage(cmdname);
+	std::vector<std::string> arg_list = argv_to_list(argc, argv);
+
+	std::string src_fn;
+	std::string dst_fn;
 	size_t w=0, h=0;
 	double res=0;
 	double origin_e=0, origin_n=0;
 	double got_en=0;
-	char *srs = NULL;
+	std::string srs;
 	double ndv=0;
 	bool got_ndv=0;
-	const char *datatype = "UINT8";
+	std::string datatype = "UINT8";
 	double affine[6];
 	bool got_affine=0;
 	char endian=0;
 
-	int argp = 1;
-	while(argp < argc) {
-		char *arg = argv[argp++];
+	size_t argp = 1;
+	while(argp < arg_list.size()) {
+		const std::string &arg = arg_list[argp++];
 		// FIXME - check for duplicate values
 		if(arg[0] == '-' && arg[1]) {
-			if(!strcmp(arg, "-wh")) {
-				char *endptr;
+			try {
+				if(arg == "-wh") {
+					if(argp == arg_list.size()) usage(cmdname);
+					w = boost::lexical_cast<int>(arg_list[argp++]);
 
-				if(argp == argc) usage(argv[0]);
-				w = (int)strtol(argv[argp++], &endptr, 10);
-				if(*endptr) usage(argv[0]);
+					if(argp == arg_list.size()) usage(cmdname);
+					h = boost::lexical_cast<int>(arg_list[argp++]);
+				} else if(arg == "-affine") {
+					for(int i=0; i<6; i++) {
+						if(argp == arg_list.size()) usage(cmdname);
+						affine[i] = boost::lexical_cast<double>(arg_list[argp++]);
+					}
+					got_affine = 1;
+				} else if(arg == "-res") {
+					if(argp == arg_list.size()) usage(cmdname);
+					res = boost::lexical_cast<double>(arg_list[argp++]);
+				} else if(arg == "-origin") {
+					if(argp == arg_list.size()) usage(cmdname);
+					origin_e = boost::lexical_cast<double>(arg_list[argp++]);
 
-				if(argp == argc) usage(argv[0]);
-				h = (int)strtol(argv[argp++], &endptr, 10);
-				if(*endptr) usage(argv[0]);
-			} else if(!strcmp(arg, "-affine")) {
-				for(int i=0; i<6; i++) {
-					if(argp == argc) usage(argv[0]);
-					char *endptr;
-					affine[i] = strtod(argv[argp++], &endptr);
-					if(*endptr) usage(argv[0]);
+					if(argp == arg_list.size()) usage(cmdname);
+					origin_n = boost::lexical_cast<double>(arg_list[argp++]);
+
+					got_en++;
+				} else if(arg == "-srs") {
+					if(argp == arg_list.size()) usage(cmdname);
+					srs = arg_list[argp++];
+				} else if(arg == "-ndv") {
+					if(argp == arg_list.size()) usage(cmdname);
+					ndv = boost::lexical_cast<double>(arg_list[argp++]);
+
+					got_ndv = 1;
+				} else if(arg == "-datatype") {
+					if(argp == arg_list.size()) usage(cmdname);
+					datatype = arg_list[argp++];
+				} else if(arg == "-lsb") {
+					endian = 'L';
+				} else if(arg == "-msb") {
+					endian = 'M';
+				} else {
+					usage(cmdname);
 				}
-				got_affine = 1;
-			} else if(!strcmp(arg, "-res")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				res = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
-			} else if(!strcmp(arg, "-origin")) {
-				char *endptr;
-
-				if(argp == argc) usage(argv[0]);
-				origin_e = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
-
-				if(argp == argc) usage(argv[0]);
-				origin_n = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
-
-				got_en++;
-			} else if(!strcmp(arg, "-srs")) {
-				if(argp == argc) usage(argv[0]);
-				srs = argv[argp++];
-			} else if(!strcmp(arg, "-ndv")) {
-				if(argp == argc) usage(argv[0]);
-				char *endptr;
-				ndv = strtod(argv[argp++], &endptr);
-				if(*endptr) usage(argv[0]);
-
-				got_ndv = 1;
-			} else if(!strcmp(arg, "-datatype")) {
-				if(argp == argc) usage(argv[0]);
-				datatype = argv[argp++];
-			} else if(!strcmp(arg, "-lsb")) {
-				endian = 'L';
-			} else if(!strcmp(arg, "-msb")) {
-				endian = 'M';
-			} else usage(argv[0]);
+			} catch(boost::bad_lexical_cast &e) {
+				fatal_error("cannot parse number given on command line");
+			}
 		} else {
-			if(src_fn && dst_fn) {
-				usage(argv[0]);
-			} else if(src_fn) {
+			if(src_fn.size() && dst_fn.size()) {
+				usage(cmdname);
+			} else if(src_fn.size()) {
 				dst_fn = arg;
 			} else {
 				src_fn = arg;
@@ -125,12 +125,12 @@ int main(int argc, char *argv[]) {
 	}
 
 	if(!(
-		src_fn && dst_fn &&
-		w && h && srs
-	)) usage(argv[0]);
+		src_fn.size() && dst_fn.size() &&
+		w && h && srs.size()
+	)) usage(cmdname);
 
 	if(!got_affine) {
-		if(!(res && got_en)) usage(argv[0]);
+		if(!(res && got_en)) usage(cmdname);
 		affine[0] = origin_e;
 		affine[1] = res;
 		affine[2] = 0;
@@ -141,25 +141,25 @@ int main(int argc, char *argv[]) {
 
 	GDALDataType gdal_dt = GDT_Unknown;
 	int bytes_per_pixel = 0;
-	if(!strcmp(datatype, "UINT8")) {
+	if(datatype == "UINT8") {
 		gdal_dt = GDT_Byte;
 		bytes_per_pixel = 1;
-	} else if(!strcmp(datatype, "UINT16")) {
+	} else if(datatype == "UINT16") {
 		gdal_dt = GDT_UInt16;
 		bytes_per_pixel = 2;
-	} else if(!strcmp(datatype, "INT16")) {
+	} else if(datatype == "INT16") {
 		gdal_dt = GDT_Int16;
 		bytes_per_pixel = 2;
-	} else if(!strcmp(datatype, "UINT32")) {
+	} else if(datatype == "UINT32") {
 		gdal_dt = GDT_UInt32;
 		bytes_per_pixel = 4;
-	} else if(!strcmp(datatype, "INT32")) {
+	} else if(datatype == "INT32") {
 		gdal_dt = GDT_Int32;
 		bytes_per_pixel = 4;
-	} else if(!strcmp(datatype, "FLOAT32")) {
+	} else if(datatype == "FLOAT32") {
 		gdal_dt = GDT_Float32;
 		bytes_per_pixel = 4;
-	} else if(!strcmp(datatype, "FLOAT64")) {
+	} else if(datatype == "FLOAT64") {
 		gdal_dt = GDT_Float64;
 		bytes_per_pixel = 8;
 	}
@@ -179,10 +179,10 @@ int main(int argc, char *argv[]) {
 	//////////// open input
 
 	FILE *fin;
-	if(!strcmp(src_fn, "-")) {
+	if(src_fn == "-") {
 		fin = stdin;
 	} else {
-		fin = fopen(src_fn, "r");
+		fin = fopen(src_fn.c_str(), "r");
 	}
 	if(!fin) fatal_error("could not open input");
 
@@ -192,13 +192,13 @@ int main(int argc, char *argv[]) {
 
 	GDALDriverH dst_driver = GDALGetDriverByName("GTiff");
 	if(!dst_driver) fatal_error("unrecognized output format (GTiff)");
-	GDALDatasetH dst_ds = GDALCreate(dst_driver, dst_fn, w, h, 1, gdal_dt, NULL);
+	GDALDatasetH dst_ds = GDALCreate(dst_driver, dst_fn.c_str(), w, h, 1, gdal_dt, NULL);
 	if(!dst_ds) fatal_error("couldn't create dst_dataset");
 
 	GDALSetGeoTransform(dst_ds, affine);
 
 	OGRSpatialReference output_srs;
-	if(output_srs.SetFromUserInput(srs) != OGRERR_NONE) {
+	if(output_srs.SetFromUserInput(srs.c_str()) != OGRERR_NONE) {
 		fatal_error("could not parse SRS");
 	}
 	char *wkt = NULL;
@@ -214,16 +214,16 @@ int main(int argc, char *argv[]) {
 
 	//////////// transfer data
 
-	uint8_t *linebuf = MYALLOC(uint8_t, w * bytes_per_pixel);
+	std::vector<uint8_t> linebuf(w * bytes_per_pixel);
 	for(size_t row=0; row<h; row++) {
 		GDALTermProgress((double)row / h, NULL, NULL);
-		if(w != fread(linebuf, bytes_per_pixel, w, fin)) {
+		if(w != fread(&linebuf[0], bytes_per_pixel, w, fin)) {
 			fatal_error("input was short");
 		}
 		if(endian_mismatch) {
-			GDALSwapWords(linebuf, bytes_per_pixel, w, 0);
+			GDALSwapWords(&linebuf[0], bytes_per_pixel, w, 0);
 		}
-		GDALRasterIO(dst_band, GF_Write, 0, row, w, 1, linebuf, w, 1, gdal_dt, 0, 0);
+		GDALRasterIO(dst_band, GF_Write, 0, row, w, 1, &linebuf[0], w, 1, gdal_dt, 0, 0);
 	}
 
 	//////////// shutdown
@@ -235,7 +235,7 @@ int main(int argc, char *argv[]) {
 	// This error is checked after the output is closed.
 	// The script exits with error but the output is
 	// still saved to disk.
-	if(fread(linebuf, 1, 1, fin)) {
+	if(fread(&linebuf[0], 1, 1, fin)) {
 		fatal_error("warning: input had extra data at end\n");
 	}
 
