@@ -27,25 +27,49 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
-#ifndef GEOCODE_H
-#define GEOCODE_H
+#ifndef DANGDAL_GEOCODE_H
+#define DANGDAL_GEOCODE_H
 
-typedef struct {
-	char *s_srs;
-	char *geo_srs;
+#include <string>
+#include <vector>
+
+namespace dangdal {
+
+struct GeoOpts {
+	static void printUsage();
+	explicit GeoOpts(std::vector<std::string> &arg_list);
+
+	std::string s_srs;
+	std::string geo_srs;
 	size_t w, h;
 	bool got_ll_en;
 	bool got_ul_en;
 	double given_left_e, given_lower_n, given_upper_n;
 	double res_x, res_y;
-} geo_opts_t;
+};
 
-typedef struct {
-	char *s_srs;
-	char *geo_srs;
+class GeoRef {
+public:
+	GeoRef(GeoOpts opt, const GDALDatasetH ds);
+
+	bool hasAffine() const { return !fwd_affine.empty(); }
+
+	void xy2en(double xpos, double ypos, double *e_out, double *n_out) const;
+	void en2xy(double east, double north, double *x_out, double *y_out) const;
+	bool en2ll(double east, double north, double *lon_out, double *lat_out) const;
+	bool ll2en(double lon, double lat, double *e_out, double *n_out) const;
+	bool xy2ll(double x, double y, double *lon_out, double *lat_out) const;
+	bool ll2xy(double lon, double lat, double *x_out, double *y_out) const;
+	void en2ll_or_die(double east, double north, double *lon_out, double *lat_out) const;
+	void ll2en_or_die(double lon, double lat, double *e_out, double *n_out) const;
+	void xy2ll_or_die(double x, double y, double *lon_out, double *lat_out) const;
+	void ll2xy_or_die(double lon, double lat, double *x_out, double *y_out) const;
+
+	std::string s_srs;
+	std::string geo_srs;
 	double res_x, res_y; // zero if there is rotation
 	double res_meters_x, res_meters_y; // zero if rotated/unknown
-	char *units_name;
+	std::string units_name;
 	double units_val;
 	bool have_semi_major;
 	double semi_major;
@@ -54,24 +78,11 @@ typedef struct {
 	OGRSpatialReferenceH geo_sref;
 	OGRCoordinateTransformationH fwd_xform;
 	OGRCoordinateTransformationH inv_xform;
-	double *fwd_affine;
-	double *inv_affine;
+	std::vector<double> fwd_affine;
+	std::vector<double> inv_affine;
 	double lon_range1, lon_range2, lon_loopsize;
-} georef_t;
+};
 
-void print_georef_usage();
-geo_opts_t init_geo_options(int *argc_ptr, char ***argv_ptr);
-georef_t init_georef(geo_opts_t *opt, GDALDatasetH ds);
+} // namespace dangdal
 
-void xy2en(georef_t *georef, double xpos, double ypos, double *e_out, double *n_out);
-void en2xy(georef_t *georef, double east, double north, double *x_out, double *y_out);
-bool en2ll(georef_t *georef, double east, double north, double *lon_out, double *lat_out);
-bool ll2en(georef_t *georef, double lon, double lat, double *e_out, double *n_out);
-bool xy2ll(georef_t *georef, double x, double y, double *lon_out, double *lat_out);
-bool ll2xy(georef_t *georef, double lon, double lat, double *x_out, double *y_out);
-void en2ll_or_die(georef_t *georef, double east, double north, double *lon_out, double *lat_out);
-void ll2en_or_die(georef_t *georef, double lon, double lat, double *e_out, double *n_out);
-void xy2ll_or_die(georef_t *georef, double x, double y, double *lon_out, double *lat_out);
-void ll2xy_or_die(georef_t *georef, double lon, double lat, double *x_out, double *y_out);
-
-#endif // ifndef GEOCODE_H
+#endif // ifndef DANGDAL_GEOCODE_H
