@@ -26,12 +26,17 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
-#ifndef MASK_TRACER_H
+#ifndef DANGDAL_MASK_TRACER_H
+#define DANGDAL_MASK_TRACER_H
 
 #include "mask.h"
 #include "polygon.h"
 
-// this function has the side effect of erasing the mask
-mpoly_t trace_mask(BitGrid mask, size_t w, size_t h, long min_area, int no_donuts);
+namespace dangdal {
 
-#endif // ifndef MASK_TRACER_H
+// this function has the side effect of erasing the mask
+Mpoly trace_mask(BitGrid &mask, size_t w, size_t h, int64_t min_area, bool no_donuts);
+
+} // namespace dangdal
+
+#endif // ifndef DANGDAL_MASK_TRACER_H
