@@ -26,13 +26,17 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
+#include <vector>
+
 #include "common.h"
 #include "mask.h"
 #include "polygon.h"
 #include "debugplot.h"
 #include "ndv.h"
 
-uint8_t *read_dataset_8bit(GDALDatasetH ds, int band_idx, uint8_t *usage_array, report_image_t *dbuf) {
+namespace dangdal {
+
+std::vector<uint8_t> read_dataset_8bit(GDALDatasetH ds, int band_idx, uint8_t *usage_array, DebugPlot *dbuf) {
 	for(int i=0; i<256; i++) usage_array[i] = 0;
 
 	size_t w = GDALGetRasterXSize(ds);
@@ -59,8 +63,8 @@ uint8_t *read_dataset_8bit(GDALDatasetH ds, int band_idx, uint8_t *usage_array,
 
 	printf("Reading one band of size %zd x %zd\n", w, h);
 
-	uint8_t *outbuf = MYALLOC(uint8_t, w*h);
-	uint8_t *inbuf = MYALLOC(uint8_t, blocksize_x*blocksize_y);
+	std::vector<uint8_t> outbuf(w*h);
+	std::vector<uint8_t> inbuf(blocksize_x*blocksize_y);
 	for(size_t boff_y=0; boff_y<h; boff_y+=blocksize_y) {
 		size_t bsize_y = blocksize_y;
 		if(bsize_y + boff_y > h) bsize_y = h - boff_y;
@@ -76,13 +80,13 @@ uint8_t *read_dataset_8bit(GDALDatasetH ds, int band_idx, uint8_t *usage_array,
 			GDALTermProgress(progress, NULL, NULL);
 
 			GDALRasterIO(band, GF_Read, boff_x, boff_y, bsize_x, bsize_y, 
-				inbuf, bsize_x, bsize_y, GDT_Byte, 0, 0);
+				&inbuf[0], bsize_x, bsize_y, GDT_Byte, 0, 0);
 
-			uint8_t *p_in = inbuf;
+			uint8_t *p_in = &inbuf[0];
 			for(size_t j=0; j<bsize_y; j++) {
 				size_t y = j + boff_y;
 				bool is_dbuf_stride_y = dbuf && ((y % dbuf->stride_y) == 0);
-				uint8_t *p_out = outbuf + w*y + boff_x;
+				uint8_t *p_out = &outbuf[w*y + boff_x];
 				for(size_t i=0; i<bsize_x; i++) {
 					uint8_t val = *(p_in++);
 					*(p_out++) = val;
@@ -95,34 +99,34 @@ uint8_t *read_dataset_8bit(GDALDatasetH ds, int band_idx, uint8_t *usage_array,
 						if(db_v < 50) db_v = 50;
 						if(db_v > 254) db_v = 254;
 						uint8_t r = (uint8_t)(db_v*.75);
-						plot_point(dbuf, x, y, r, db_v, db_v);
+						dbuf->plotPoint(x, y, r, db_v, db_v);
 					}
 				}
 			}
 		}
 	}
 
-	free(inbuf);
-
 	GDALTermProgress(1, NULL, NULL);
 
 	return outbuf;
 }
 
-BitGrid get_bitgrid_for_dataset(GDALDatasetH ds, int bandlist_size, int *bandlist, 
-ndv_def_t *ndv_def, report_image_t *dbuf) {
+BitGrid get_bitgrid_for_dataset(
+	GDALDatasetH ds, const std::vector<size_t> &bandlist,
+	const NdvDef &ndv_def, DebugPlot *dbuf
+) {
 	size_t w = GDALGetRasterXSize(ds);
 	size_t h = GDALGetRasterYSize(ds);
-	int band_count = GDALGetRasterCount(ds);
-	if(VERBOSE) printf("input is %zd x %zd x %d\n", w, h, band_count);
+	size_t band_count = GDALGetRasterCount(ds);
+	if(VERBOSE) printf("input is %zd x %zd x %zd\n", w, h, band_count);
 
 	BitGrid mask(w, h);
 	mask.zero();
 
-	printf("Reading %d bands of size %zd x %zd\n", bandlist_size, w, h);
+	printf("Reading %zd bands of size %zd x %zd\n", bandlist.size(), w, h);
 
-	for(int bandlist_idx=0; bandlist_idx<bandlist_size; bandlist_idx++) {
-		int band_idx = bandlist[bandlist_idx];
+	for(size_t bandlist_idx=0; bandlist_idx<bandlist.size(); bandlist_idx++) {
+		size_t band_idx = bandlist[bandlist_idx];
 		if(band_idx < 1 || band_idx > band_count) fatal_error("bandid out of range");
 
 		GDALRasterBandH band = GDALGetRasterBand(ds, band_idx);
@@ -136,17 +140,18 @@ ndv_def_t *ndv_def, report_image_t *dbuf) {
 		GDALDataType gdt = GDALGetRasterDataType(band);
 		bool use_8bit = (gdt == GDT_Byte);
 
-		if(VERBOSE) printf("band %d: block size = %zd,%zd, use_8bit=%d\n",
+		if(VERBOSE) printf("band %zd: block size = %zd,%zd, use_8bit=%d\n",
 			band_idx, blocksize_x, blocksize_y, use_8bit?1:0);
 
-		void *block_buf;
+		std::vector<uint8_t> block_buf_8bit;
+		std::vector<double> block_buf_dbl;
 		if(use_8bit) {
-			block_buf = MYALLOC(uint8_t, blocksize_x*blocksize_y);
+			block_buf_8bit.resize(blocksize_x*blocksize_y);
 		} else {
-			block_buf = MYALLOC(double, blocksize_x*blocksize_y);
+			block_buf_dbl.resize(blocksize_x*blocksize_y);
 		}
 
-		uint8_t *row_ndv = MYALLOC(uint8_t, blocksize_x);
+		std::vector<uint8_t> row_ndv(blocksize_x);
 
 		for(size_t boff_y=0; boff_y<h; boff_y+=blocksize_y) {
 			size_t bsize_y = blocksize_y;
@@ -160,26 +165,33 @@ ndv_def_t *ndv_def, report_image_t *dbuf) {
 						bandlist_idx * w * h +
 						boff_y * w +
 						boff_x * bsize_y
-					) / (bandlist_size * w * h);
+					) / (bandlist.size() * w * h);
 				GDALTermProgress(progress, NULL, NULL);
 
-				GDALRasterIO(band, GF_Read, boff_x, boff_y, bsize_x, bsize_y, 
-					block_buf, bsize_x, bsize_y, 
-					use_8bit ? GDT_Byte : GDT_Float64,
-					0, 0);
+				if(use_8bit) {
+					GDALRasterIO(band, GF_Read, boff_x, boff_y, bsize_x, bsize_y, 
+						&block_buf_8bit[0], bsize_x, bsize_y, GDT_Byte, 0, 0);
+				} else {
+					GDALRasterIO(band, GF_Read, boff_x, boff_y, bsize_x, bsize_y, 
+						&block_buf_dbl[0], bsize_x, bsize_y, GDT_Float64, 0, 0);
+				}
 
 				double *p_dbl = NULL;
 				uint8_t *p_8bit = NULL;
 				if(use_8bit) {
-					p_8bit = (uint8_t *)block_buf;
+					p_8bit = &block_buf_8bit[0];
 				} else {
-					p_dbl = (double *)block_buf;
+					p_dbl = &block_buf_dbl[0];
 				}
 				for(size_t j=0; j<bsize_y; j++) {
 					size_t y = j + boff_y;
 					bool is_dbuf_stride_y = dbuf && (bandlist_idx==0) && ((y % dbuf->stride_y) == 0);
 
-					array_check_ndv(ndv_def, bandlist_idx, p_dbl, p_8bit, row_ndv, bsize_x);
+					if(p_dbl) {
+						ndv_def.arrayCheckNdv(bandlist_idx, p_dbl, &row_ndv[0], bsize_x);
+					} else {
+						ndv_def.arrayCheckNdv(bandlist_idx, p_8bit, &row_ndv[0], bsize_x);
+					}
 
 					for(size_t i=0; i<bsize_x; i++) {
 						bool is_dbuf_stride = is_dbuf_stride_y && ((i % dbuf->stride_x) == 0);
@@ -190,7 +202,7 @@ ndv_def_t *ndv_def, report_image_t *dbuf) {
 							if(db_v < 50) db_v = 50;
 							if(db_v > 254) db_v = 254;
 							uint8_t r = (uint8_t)(db_v*.75);
-							plot_point(dbuf, x, y, r, (uint8_t)db_v, (uint8_t)db_v);
+							dbuf->plotPoint(x, y, r, (uint8_t)db_v, (uint8_t)db_v);
 						}
 
 						if(use_8bit) p_8bit++;
@@ -201,7 +213,7 @@ ndv_def_t *ndv_def, report_image_t *dbuf) {
 						for(size_t i=0; i<bsize_x; i++) {
 							mask.set(boff_x+i, y, !row_ndv[i]);
 						}
-					} else if(ndv_def->invert) {
+					} else if(ndv_def.isInvert()) {
 						for(size_t i=0; i<bsize_x; i++) {
 							if(row_ndv[i]) mask.set(boff_x+i, y, false);
 						}
@@ -213,16 +225,13 @@ ndv_def_t *ndv_def, report_image_t *dbuf) {
 				}
 			}
 		}
-
-		free(block_buf);
-		free(row_ndv);
 	}
 
 	if(dbuf) {
 		for(size_t y=0; y<h; y+=dbuf->stride_y) {
 		for(size_t x=0; x<w; x+=dbuf->stride_x) {
 			if(!mask(x, y)) {
-				plot_point(dbuf, x, y, 0, 0, 0);
+				dbuf->plotPoint(x, y, 0, 0, 0);
 			}
 		}
 		}
@@ -287,8 +296,8 @@ void BitGrid::erode() { // FIXME - untested
 	delete[] rowl;
 }
 
-vertex_t BitGrid::centroid() {
-	long accum_x=0, accum_y=0, cnt=0;
+Vertex BitGrid::centroid() {
+	int64_t accum_x=0, accum_y=0, cnt=0;
 	for(int y=0; y<h; y++) {
 		for(int x=0; x<w; x++) {
 			if(get(x, y)) {
@@ -298,8 +307,10 @@ vertex_t BitGrid::centroid() {
 			}
 		}
 	}
-	return (vertex_t){
+	return Vertex(
 		double(accum_x) / cnt,
 		double(accum_y) / cnt
-	};
+	);
 }
+
+} // namespace dangdal
