@@ -27,22 +27,25 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
-#ifndef MASK_H
-#define MASK_H
+#ifndef DANGDAL_MASK_H
+#define DANGDAL_MASK_H
+
+#include <cassert>
+#include <vector>
 
-#include <boost/noncopyable.hpp>
-#include <boost/smart_ptr.hpp>
 #include "common.h"
 #include "polygon.h"
 #include "debugplot.h"
 #include "ndv.h"
 
+namespace dangdal {
+
 class BitGrid {
 public:
 	BitGrid(int _w, int _h) :
 		w(_w), h(_h),
 		arrlen((size_t(w)*h+7)/8),
-		grid(new uint8_t[arrlen])
+		grid(arrlen)
 	{ }
 
 // default dtor, copy, assign are OK
@@ -85,17 +88,19 @@ public:
 
 	void erode();
 
-	vertex_t centroid();
+	Vertex centroid();
 
 private:
 	int w, h;
 	size_t arrlen;
-	boost::shared_array<uint8_t> grid;
+	std::vector<uint8_t> grid;
 };
 
-uint8_t *read_dataset_8bit(GDALDatasetH ds, int band_idx, uint8_t *usage_array, report_image_t *dbuf);
-BitGrid get_bitgrid_for_dataset(GDALDatasetH ds, int bandlist_size, int *bandlist, 
-	ndv_def_t *ndv_def, report_image_t *dbuf);
+std::vector<uint8_t> read_dataset_8bit(GDALDatasetH ds, int band_idx, uint8_t *usage_array, DebugPlot *dbuf);
+BitGrid get_bitgrid_for_dataset(GDALDatasetH ds, const std::vector<size_t> &bandlist, 
+	const NdvDef &ndv_def, DebugPlot *dbuf);
 BitGrid get_bitgrid_for_8bit_raster(size_t w, size_t h, const uint8_t *raster, uint8_t wanted);
 
-#endif // ifndef MASK_H
+} // namespace dangdal
+
+#endif // ifndef DANGDAL_MASK_H
