@@ -26,20 +26,27 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
-#ifndef POLYGON_RASTERIZER_H
-#define POLYGON_RASTERIZER_H
+#ifndef DANGDAL_POLYGON_RASTERIZER_H
+#define DANGDAL_POLYGON_RASTERIZER_H
+
+#include <string>
+#include <vector>
 
 #include "common.h"
+#include "polygon.h"
 
-typedef struct {
-	int num_crossings;
-	int array_size;
-	int *crossings;
-} row_crossings_t;
+namespace dangdal {
 
-row_crossings_t *get_row_crossings(mpoly_t *mpoly, int min_y, int num_rows);
-void free_row_crossings(row_crossings_t *rc, int num_rows);
-void mask_from_mpoly(mpoly_t *mpoly, size_t w, size_t h, const char *fn);
-void crossings_intersection(row_crossings_t *out, row_crossings_t *in1, row_crossings_t *in2);
+typedef std::vector<int> row_crossings_t;
 
-#endif // ifndef POLYGON_RASTERIZER_H
+std::vector<row_crossings_t> get_row_crossings(const Mpoly &mpoly, int min_y, int num_rows);
+
+void mask_from_mpoly(const Mpoly &mpoly, size_t w, size_t h, const std::string fn);
+
+row_crossings_t crossings_intersection(
+	const row_crossings_t &in1, const row_crossings_t &in2
+);
+
+} // namespace dangdal
+
+#endif // ifndef DANGDAL_POLYGON_RASTERIZER_H
