@@ -32,30 +32,32 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 #include "debugplot.h"
 #include "mask.h"
 
-typedef struct {
-	vertex_t p0, p1;
+namespace dangdal {
+
+struct Edge {
+	Vertex p0, p1;
 	double angle;
 	double seg_len;
 	int group;
-} edge_t;
+};
 
-typedef struct {
+struct EdgeGroup {
 	double arc_len;
 	double wx, wy;
 	double avg_ang;
 	int use;
-	edge_t best_edge;
+	Edge best_edge;
 	double sort_key;
-} edge_group_t;
+};
 
 double ang_diff(double a1, double a2) {
 	double d = fabs(a1 - a2);
 	return d<=180.0 ? d : 360.0-d;
 }
 
-ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *dbuf) {
-	int *chrows_left = MYALLOC(int, h);
-	int *chrows_right = MYALLOC(int, h);
+Ring calc_rect4_from_convex_hull(const BitGrid &mask, int w, int h, DebugPlot *dbuf) {
+	std::vector<int> chrows_left(h);
+	std::vector<int> chrows_right(h);
 	for(int j=0; j<h; j++) {
 		chrows_left[j] = w;
 		chrows_right[j] = -1;
@@ -84,12 +86,11 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 	if(fulcrum_x<0) fatal_error("image was empty");
 	//if(VERBOSE) printf("start point: %d,%d\n", fulcrum_x, fulcrum_y);
 
-	edge_t *all_edges = NULL;
-	int num_edges = 0;
+	std::vector<Edge> all_edges;
 
 	int chop_dx = 1, chop_dy = 0;
 	for(;;) {
-		if(dbuf && dbuf->mode == PLOT_RECT4) plot_point_big(dbuf, fulcrum_x, fulcrum_y, 0, 255, 0);
+		if(dbuf && dbuf->mode == PLOT_RECT4) dbuf->plotPointBig(fulcrum_x, fulcrum_y, 0, 255, 0);
 
 		int best_dx = -chop_dx, best_dy = -chop_dy;
 		int best_x=-1, best_y=-1;
@@ -117,12 +118,12 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 		//if(VERBOSE) printf("  f=[%3d,%3d] ", best_x, best_y);
 		//if(VERBOSE) printf("  a=[% 3.1f]\n", angle);
 
-		all_edges = REMYALLOC(edge_t, all_edges, (num_edges+1));
-		all_edges[num_edges].p0.x = fulcrum_x;
-		all_edges[num_edges].p0.y = fulcrum_y;
-		all_edges[num_edges].p1.x = best_x;
-		all_edges[num_edges].p1.y = best_y;
-		num_edges++;
+		Edge new_edge;
+		new_edge.p0.x = fulcrum_x;
+		new_edge.p0.y = fulcrum_y;
+		new_edge.p1.x = best_x;
+		new_edge.p1.y = best_y;
+		all_edges.push_back(new_edge);
 
 		if(best_x<0) fatal_error("could not find new fulcrum");
 		fulcrum_x = best_x; fulcrum_y = best_y;
@@ -130,8 +131,10 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 		chop_dx = best_dx; chop_dy = best_dy;
 	}
 
-	for(int i=0; i<num_edges; i++) {
-		edge_t e = all_edges[i];
+	const size_t num_edges = all_edges.size();
+
+	for(size_t i=0; i<num_edges; i++) {
+		Edge e = all_edges[i];
 		double dx = e.p1.x - e.p0.x;
 		double dy = e.p1.y - e.p0.y;
 		e.angle = atan2(dy, dx)*180.0/M_PI;
@@ -146,9 +149,9 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 
 	int num_groups = 0;
 	all_edges[0].group = (num_groups++);
-	for(int i=0; i<num_edges; i++) {
-		edge_t l = all_edges[i];
-		edge_t r = all_edges[(i+1) % num_edges];
+	for(size_t i=0; i<num_edges; i++) {
+		Edge l = all_edges[i];
+		Edge r = all_edges[(i+1) % num_edges];
 		double len = l.seg_len + r.seg_len;
 		double adiff = ang_diff(l.angle, r.angle);
 
@@ -160,7 +163,7 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 			} else {
 				// wrapped around... set the tail group to be
 				// equal to the head group
-				int j;
+				size_t j;
 				for(j=num_edges-1; j; j--) {
 					if(all_edges[j].group != l.group) {
 						j++; break;
@@ -187,15 +190,15 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 		}
 	}
 	if(VERBOSE) printf("num groups: %d\n", num_groups);
-	for(int i=0; i<num_edges; i++) {
+	for(size_t i=0; i<num_edges; i++) {
 		if(all_edges[i].group < 0) fatal_error("edge not assigned to a group");
 		//all_edges[i].group = (num_groups++);
 	}
 	//if(VERBOSE) printf("num groups: %d\n", num_groups);
 
-	if(VERBOSE) for(int i=0; i<num_edges; i++) {
-		edge_t l = all_edges[i];
-		edge_t r = all_edges[(i+1) % num_edges];
+	if(VERBOSE) for(size_t i=0; i<num_edges; i++) {
+		Edge l = all_edges[i];
+		Edge r = all_edges[(i+1) % num_edges];
 		double len = l.seg_len + r.seg_len;
 		double adiff = ang_diff(l.angle, r.angle);
 		printf("a=%.15f  l=%.15f  ", l.angle, l.seg_len);
@@ -203,17 +206,17 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 		printf("  group=%d\n", l.group);
 	}
 
-	edge_group_t *groups = MYALLOC(edge_group_t, num_groups);
+	std::vector<EdgeGroup> groups(num_groups);
 	for(int i=0; i<num_groups; i++) {
 		groups[i].arc_len = 0;
 		groups[i].wx = 0;
 		groups[i].wy = 0;
 	}
-	for(int i=0; i<num_edges; i++) {
-		edge_t e = all_edges[i];
+	for(size_t i=0; i<num_edges; i++) {
+		Edge e = all_edges[i];
 		int eg = e.group;
 		if(eg < 0 || eg >= num_groups) {
-			fatal_error("group out of range (i=%d, g=%d, num_groups=%d)", i, eg, num_groups);
+			fatal_error("group out of range (i=%zd, g=%d, num_groups=%d)", i, eg, num_groups);
 		}
 		groups[eg].arc_len += e.seg_len;
 		groups[eg].wx += e.seg_len * cos(e.angle / 180.0 * M_PI);
@@ -244,7 +247,7 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 		// be better to create a new edge with the desired angle and with proper
 		// p0.x,p0.y value
 		groups[i].best_edge = all_edges[0];
-		for(int j=0; j<num_edges; j++) {
+		for(size_t j=0; j<num_edges; j++) {
 			double d1 = ang_diff(groups[i].avg_ang, all_edges[j].angle);
 			double d2 = ang_diff(groups[i].avg_ang, groups[i].best_edge.angle);
 			if(d1 < d2) groups[i].best_edge = all_edges[j];
@@ -261,8 +264,8 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 	// bubble sort - start at top edge and go clockwise
 	for(int i=0; i<num_groups; i++) {
 		for(int j=num_groups-1; j>i; j--) {
-			edge_group_t e1 = groups[j-1];
-			edge_group_t e2 = groups[j];
+			EdgeGroup e1 = groups[j-1];
+			EdgeGroup e2 = groups[j];
 			if(e1.sort_key > e2.sort_key) {
 				groups[j-1] = e2;
 				groups[j] = e1;
@@ -275,37 +278,34 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 	}
 
 	//if(VERBOSE) printf("%d edges\n", num_groups);
-	vertex_t *verts = MYALLOC(vertex_t, num_groups);
+	std::vector<Vertex> verts(num_groups);
 	for(int i=0; i<num_groups; i++) {
 		int j = i ? i-1 : num_groups-1;
-		edge_t e1 = groups[i].best_edge;
-		edge_t e2 = groups[j].best_edge;
-		line_line_intersection(
+		Edge e1 = groups[i].best_edge;
+		Edge e2 = groups[j].best_edge;
+		verts[i] = line_line_intersection(
 			e1.p0, e1.p1,
-			e2.p0, e2.p1,
-			&verts[i]);
+			e2.p0, e2.p1);
 		if(VERBOSE) printf("vert[%d] = %.15f, %.15f\n", i, verts[i].x, verts[i].y);
 	}
 
 	if(dbuf && dbuf->mode == PLOT_RECT4) {
 		for(int i=0; i<num_groups; i++) {
 			int j = i<num_groups-1 ? i+1 : 0;
-			plot_line(dbuf, verts[i], verts[j], 255, 0, 0);
-			plot_point_big(dbuf, verts[i].x, verts[i].y, 255, 255, 0);
-			plot_point_big(dbuf, verts[j].x, verts[j].y, 255, 255, 0);
+			dbuf->plotLine(verts[i], verts[j], 255, 0, 0);
+			dbuf->plotPointBig(verts[i].x, verts[i].y, 255, 255, 0);
+			dbuf->plotPointBig(verts[j].x, verts[j].y, 255, 255, 0);
 		}
 	}
 
-	ring_t rect4;
-	if(num_groups == 4) {
-		rect4.npts = 4;
+	if(verts.size() == 4) {
+		Ring rect4;
 		rect4.pts = verts;
+		return rect4;
 	} else {
-		rect4.npts = 0;
-		rect4.pts = NULL;
 		printf("could not find a 4-sided bounding polygon\n");
+		return Ring();
 	}
-	return rect4;
 }
 
 //static int is_in_crossings(row_crossings_t *r, int x) {
@@ -317,37 +317,36 @@ ring_t calc_rect4_from_convex_hull(BitGrid mask, int w, int h, report_image_t *d
 //	return v;
 //}
 
-static int ringdiff(ring_t *r1, ring_t *r2, BitGrid mask) {
-	mpoly_t mp1, mp2;
-	mp1.num_rings = mp2.num_rings = 1;
-	mp1.rings = r1;
-	mp2.rings = r2;
+static int ringdiff(const Ring &r1, const Ring &r2, const BitGrid &mask) {
+	Mpoly mp1, mp2;
+	mp1.rings.push_back(r1);
+	mp2.rings.push_back(r2);
 
-	bbox_t bb = union_bbox(get_ring_bbox(r1), get_ring_bbox(r2));
+	Bbox bb = box_union(r1.getBbox(), r2.getBbox());
 	//int min_x = (int)floor(bb.min_x);
 	int min_y = (int)floor(bb.min_y);
 	int max_x = (int)ceil (bb.max_x);
 	int max_y = (int)ceil (bb.max_y);
 
-	row_crossings_t *rcs1 = get_row_crossings(&mp1, min_y, max_y-min_y+1);
-	row_crossings_t *rcs2 = get_row_crossings(&mp2, min_y, max_y-min_y+1);
+	std::vector<row_crossings_t> rcs1 = get_row_crossings(mp1, min_y, max_y-min_y+1);
+	std::vector<row_crossings_t> rcs2 = get_row_crossings(mp2, min_y, max_y-min_y+1);
 
 	int tally = 0;
 	for(int y=min_y; y<=max_y; y++) {
-		row_crossings_t *row1 = rcs1 + y - min_y;
-		row_crossings_t *row2 = rcs2 + y - min_y;
+		const row_crossings_t &row1 = rcs1[y - min_y];
+		const row_crossings_t &row2 = rcs2[y - min_y];
 
-		int in1=0, in2=0;
-		int ci1=0, ci2=0;
+		bool in1=0, in2=0;
+		size_t ci1=0, ci2=0;
 		for(;;) {
-			int cx1 = ci1 < row1->num_crossings ? row1->crossings[ci1] : max_x+1;
-			int cx2 = ci2 < row2->num_crossings ? row2->crossings[ci2] : max_x+1;
+			int cx1 = ci1 < row1.size() ? row1[ci1] : max_x+1;
+			int cx2 = ci2 < row2.size() ? row2[ci2] : max_x+1;
 			// FIXME
 			//if(cx1 > max_x+1 || cx2 > max_x+1) fatal_error("cx > max_x+1 (%d,%d,%d)", cx1, cx2, max_x+1);
 			//if(cx1 == max_x+1 && cx2 == max_x+1) break;
 			if(cx1 >= max_x+1 && cx2 >= max_x+1) break;
 
-			int x_from = MIN(cx1, cx2);
+			int x_from = std::min(cx1, cx2);
 			if(cx1 < cx2) {
 				in1 = !in1;
 				ci1++;
@@ -358,9 +357,9 @@ static int ringdiff(ring_t *r1, ring_t *r2, BitGrid mask) {
 
 			if((in1 && in2) || (!in1 && !in2)) continue;
 
-			cx1 = ci1 < row1->num_crossings ? row1->crossings[ci1] : max_x+1;
-			cx2 = ci2 < row2->num_crossings ? row2->crossings[ci2] : max_x+1;
-			int x_to = MIN(cx1, cx2);
+			cx1 = ci1 < row1.size() ? row1[ci1] : max_x+1;
+			cx2 = ci2 < row2.size() ? row2[ci2] : max_x+1;
+			int x_to = std::min(cx1, cx2);
 			
 			int gain=1, penalty=2; // FIXME - arbitrary
 			for(int x=x_from; x<x_to; x++) {
@@ -380,8 +379,6 @@ static int ringdiff(ring_t *r1, ring_t *r2, BitGrid mask) {
 		//	}
 		//}
 	}
-	free_row_crossings(rcs1, max_y-min_y+1);
-	free_row_crossings(rcs2, max_y-min_y+1);
 	return tally;
 }
 
@@ -399,30 +396,33 @@ static int rand_delta(int amt) {
 	return rand_range(-amt, amt);
 }
 
-static void perturb(ring_t *in, ring_t *out, int amt) {
+static void perturb(const Ring &in, Ring &out, int amt) {
+	assert(in.pts.size() == 4);
+	assert(out.pts.size() == 4);
+
 	int parallelogram = 1;
 	if(parallelogram) {
 		for(int v=0; v<3; v++) {
-			out->pts[v] = in->pts[v];
-			out->pts[v].x += rand_delta(amt);
-			out->pts[v].y += rand_delta(amt);
+			out.pts[v] = in.pts[v];
+			out.pts[v].x += rand_delta(amt);
+			out.pts[v].y += rand_delta(amt);
 		}
-		out->pts[3].x = out->pts[0].x + out->pts[2].x - out->pts[1].x;
-		out->pts[3].y = out->pts[0].y + out->pts[2].y - out->pts[1].y;
+		out.pts[3].x = out.pts[0].x + out.pts[2].x - out.pts[1].x;
+		out.pts[3].y = out.pts[0].y + out.pts[2].y - out.pts[1].y;
 	} else {
 		int corner = rand_range(0, 3);
 		for(int v=0; v<4; v++) {
-			out->pts[v] = in->pts[v];
+			out.pts[v] = in.pts[v];
 			if(v == corner) {
-				out->pts[v].x += rand_delta(amt);
-				out->pts[v].y += rand_delta(amt);
+				out.pts[v].x += rand_delta(amt);
+				out.pts[v].y += rand_delta(amt);
 			}
 		}
 	}
 }
 
 /*
-static ring_t anneal(ring_t *input, int recurse, BitGrid mask, int w, int h) {
+static ring_t anneal(ring_t *input, int recurse, const BitGrid &mask, int w, int h) {
 	ring_t best = duplicate_ring(input);
 	ring_t pert = duplicate_ring(input);
 
@@ -464,39 +464,41 @@ static ring_t anneal(ring_t *input, int recurse, BitGrid mask, int w, int h) {
 }
 */
 
-static ring_t anneal(ring_t *input, BitGrid mask) {
-	ring_t best = duplicate_ring(input);
-	ring_t pert = duplicate_ring(input);
+static Ring anneal(const Ring &input, const BitGrid &mask) {
+	Ring best = input;
+	Ring pert = input;
 
 	for(int iter=0; iter<10000; iter++) { // FIXME - arbitrary
 		int amt = (int)ceil(200.0 * exp(-iter / 50.0)); // FIXME - arbitrary
-		perturb(&best, &pert, amt);
-		int diff = ringdiff(&best, &pert, mask);
+		perturb(best, pert, amt);
+		int diff = ringdiff(best, pert, mask);
 		if(diff > 0) {
 			// swap best and pert
-			ring_t t = best; best = pert; pert = t;
+			Ring t = best; best = pert; pert = t;
 		}
 	}
 
-	free_ring(&pert);
 	return best;
 }
 
-ring_t calc_rect4_from_mask(BitGrid mask, int w, int h, report_image_t *dbuf, bool use_ai) {
-	ring_t best = calc_rect4_from_convex_hull(mask, w, h, dbuf);
+Ring calc_rect4_from_mask(const BitGrid &mask, int w, int h, DebugPlot *dbuf, bool use_ai) {
+	Ring best = calc_rect4_from_convex_hull(mask, w, h, dbuf);
+	if(best.pts.size() == 0) return best;
 
 	if(use_ai) {
-		best = anneal(&best, mask);
+		best = anneal(best, mask);
 
 		if(dbuf && dbuf->mode == PLOT_RECT4) {
-			for(int i=0; i<best.npts; i++) {
-				int i2 = (i+1) % best.npts;
-				plot_line(dbuf, best.pts[i], best.pts[i2], 0, 255, 0);
-				plot_point_big(dbuf, best.pts[i].x, best.pts[i].y, 255, 255, 0);
-				plot_point_big(dbuf, best.pts[i2].x, best.pts[i2].y, 255, 255, 0);
+			for(size_t i=0; i<best.pts.size(); i++) {
+				size_t i2 = (i+1) % best.pts.size();
+				dbuf->plotLine(best.pts[i], best.pts[i2], 0, 255, 0);
+				dbuf->plotPointBig(best.pts[i].x, best.pts[i].y, 255, 255, 0);
+				dbuf->plotPointBig(best.pts[i2].x, best.pts[i2].y, 255, 255, 0);
 			}
 		}
 	}
 
 	return best;
 }
+
+} // namespace dangdal
