@@ -25,6 +25,15 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 */
 
 
+#ifndef DANGDAL_RECTANGLE_FINDER_H
+#define DANGDAL_RECTANGLE_FINDER_H
+
 #include "mask.h"
 
-ring_t calc_rect4_from_mask(BitGrid mask, int w, int h, report_image_t *dbuf, bool use_ai);
+namespace dangdal {
+
+Ring calc_rect4_from_mask(const BitGrid &mask, int w, int h, DebugPlot *dbuf, bool use_ai);
+
+} // namespace dangdal
+
+#endif // DANGDAL_RECTANGLE_FINDER_H
